from __future__ import print_function

import os, sys
import gzip
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)

# additional package
import numpy

# wurp package
import table.interact_table as it
import clog as logger

################################################################################
class TopologyParser:

    """
    >>> tpl = TopologyParser('./test/ala3.prmtop')
    >>> tpl.parse()
    >>> tpl.get_molcule_info()

    """

    section_end = '%END FLAG'
    comment = ';'

    def __init__(self, filename):
        self._filename = filename
        self.__fname_to_info = {}

    def get_param_infos(self, flag_name):
        return self.__fname_to_info[flag_name]

    def parse(self):
        if self._filename.endswith('.gz'):
            tpl_file = gzip.open(self._filename, 'rb')
        else:
            tpl_file = open(self._filename, 'r')
        fname_to_lines = self.split_content(tpl_file)
        tpl_file.close()

        fname_to_parsers = {
            'ATOM_NAME'                  : self._parse_name(20, 4)    , 
            'CHARGE'                     : self._parse_generic(float) , 
            'MASS'                       : self._parse_generic(float) , 
            'ATOM_TYPE_INDEX'            : self._parse_generic(int)   , 
            'RESIDUE_LABEL'              : self._parse_name(20, 4)    , 
            'RESIDUE_POINTER'            : self._parse_generic(int)   , 
            'BOND_FORCE_CONSTANT'        : self._parse_generic(float) , # kcal/mol
            'BOND_EQUIL_VALUE'           : self._parse_generic(float) , # A
            'BONDS_INC_HYDROGEN'         : self._parse_generic(int)   , 
            'BONDS_WITHOUT_HYDROGEN'     : self._parse_generic(int)   , 
            'ANGLE_FORCE_CONSTANT'       : self._parse_generic(float) , # kcal/mol A^2
            'ANGLE_EQUIL_VALUE'          : self._parse_generic(float) , # radian
            'ANGLES_INC_HYDROGEN'        : self._parse_generic(int)   , 
            'ANGLES_WITHOUT_HYDROGEN'    : self._parse_generic(int)   , 
            'DIHEDRAL_FORCE_CONSTANT'    : self._parse_generic(float) , # kcal/mol
            'DIHEDRAL_PERIODICITY'       : self._parse_generic(float) , 
            'DIHEDRAL_PHASE'             : self._parse_generic(float) , 
            'DIHEDRALS_INC_HYDROGEN'     : self._parse_generic(int)   , 
            'DIHEDRALS_WITHOUT_HYDROGEN' : self._parse_generic(int)   , 
            'NONBONDED_PARM_INDEX'       : self._parse_generic(int)   , 
            'LENNARD_JONES_ACOEF'        : self._parse_generic(float) , 
            'LENNARD_JONES_BCOEF'        : self._parse_generic(float) , 
            'HBOND_ACOEF'                : self._parse_generic(float) , 
            'HBOND_BCOEF'                : self._parse_generic(float) , 
            'AMBER_ATOM_TYPE'            : self._parse_name(20, 4)    , 
            'BOX_DIMENSIONS'             : self._parse_generic(float) , 
        }
        for fname, parser in fname_to_parsers.items():
            if fname in fname_to_lines:
                lines = fname_to_lines[fname]
                self.__fname_to_info[fname] = list(parser(lines))
            else:
                self.__fname_to_info[fname] = None

        for fname, info in self.__fname_to_info.items():
            self.print_info(fname, info)

    def split_content(self, tpl_file):

        fname_to_lines = {}

        gen_lines = self.gen_optimized_lines(tpl_file)
        sections = []
        for line in gen_lines:
            if not self.is_section(line): continue

            flagname = line.split()[1]
            lines = list( self.gen_section_lines(gen_lines) )
            fname_to_lines[flagname] = lines 

        return fname_to_lines

    def gen_optimized_lines(self, lines):
        """Generate lines with the end of sections and without the comments."""

        # search first section marker
        for line in lines:
            line = line.split(self.comment)[0].strip()
            if self.is_section(line):
                yield line
                break

        # rest
        for line in lines:
            line = line.split(self.comment)[0].strip()
            if self.is_section(line):
                yield self.section_end
                yield line
            elif line.startswith('%FORMAT'):
                continue
            elif line != '':
                yield line
            else:
                continue
        else:
            yield self.section_end

    def is_section(self, line):
        if line == self.section_end:
            return False
        elif line.startswith('%FLAG'):
            return True
        else:
            return False

    def gen_section_lines(self, gen):
        for line in gen:
            if line == self.section_end:
                break
            yield line

    def print_info(self, header, data):
        logger.debug('['+header+']')
        logger.debug(data)

    def _parse_generic(self, convtype):
        def parse(lines):
            for line in lines:
                cols = line.split()
                for col in cols:
                    yield convtype(col)
        return parse

    def _parse_name(self, ncol=20, width=4):
        def parse(lines):
            for line in lines:
                for icol in range(ncol):
                    name = line[icol*width:(icol+1)*width].strip()
                    if name == '': continue
                    yield name
        return parse


################################################################################


# definition of vdw radius parameters.
type_to_vdws = {
        'H'  : 0.6000 , 
        # 'HO' : 0.0000 , 
        'HO' : 0.6000 ,  # Artifact
        'HS' : 0.6000 , 
        'HC' : 1.4870 , 
        'H1' : 1.3870 , 
        'H2' : 1.2870 , 
        'H3' : 1.1870 , 
        'HP' : 1.1000 , 
        'HA' : 1.4590 , 
        'H4' : 1.4090 , 
        'H5' : 1.3590 , 
        # 'HW' : 0.0000 , 
        'HW' : 0.6000 , # Artifact
        'HZ' : 1.4590 , 
        'O'  : 1.6612 , 
        'O2' : 1.6612 , 
        'OW' : 1.7683 , 
        'OH' : 1.7210 , 
        'OS' : 1.6837 , 
        'C*' : 1.9080 , 
        'CT' : 1.9080 , 
        'CX' : 1.9080 , 
        'C'  : 1.9080 , 
        'N'  : 1.8240 , 
        'N3' : 1.8240 , 
        'NY' : 1.8240 , 
        'S'  : 2.0000 , 
        'SH' : 2.0000 , 
        'P'  : 2.1000 , 
        'IM' : 2.47   , 
        'Li' : 1.1370 , 
        'IP' : 1.8680 , 
        'Na' : 1.8680 , 
        'Na+': 1.8680 , 
        'K'  : 2.6580 , 
        'Rb' : 2.9560 , 
        'Cs' : 3.3950 , 
        'MG' : 0.7926 , 
        'C0' : 1.7131 , 
        'Zn' : 1.10   , 
        'F'  : 1.75   , 
        'Cl' : 1.948  , 
        'Cl-': 1.948  , 
        'Br' : 2.22   , 
        'I'  : 2.35   , 
        'IB' : 5.0    , 
        'LP' : 0.00   , 
  }

# for gaff
gaff_type_to_vdws = {
        'h1' : 1.3870 , 
        'h2' : 1.2870 , 
        'h3' : 1.1870 , 
        'h4' : 1.4090 , 
        'h5' : 1.3590 , 
        'ha' : 1.4590 , 
        'hc' : 1.4870 , 
        'hn' : 0.6000 , 
        'ho' : 0.0000 , 
        'hp' : 0.6000 , 
        'hs' : 0.6000 , 
        'hw' : 0.0000 , 
        'hx' : 1.1000 , 
        'o'  : 1.6612 , 
        'oh' : 1.7210 , 
        'os' : 1.6837 , 
        'ow' : 1.7683 , 
        'c'  : 1.9080 , 
        'c1' : 1.9080 , 
        'c2' : 1.9080 , 
        'c3' : 1.9080 , 
        'ca' : 1.9080 , 
        'cc' : 1.9080 , 
        'cd' : 1.9080 , 
        'ce' : 1.9080 , 
        'cf' : 1.9080 , 
        'cg' : 1.9080 , 
        'ch' : 1.9080 , 
        'cp' : 1.9080 , 
        'cq' : 1.9080 , 
        'cu' : 1.9080 , 
        'cv' : 1.9080 , 
        'cx' : 1.9080 , 
        'cy' : 1.9080 , 
        'cz' : 1.9080 , 
        'n'  : 1.8240 , 
        'n1' : 1.8240 , 
        'n2' : 1.8240 , 
        'n3' : 1.8240 , 
        'n4' : 1.8240 , 
        'na' : 1.8240 , 
        'nb' : 1.8240 , 
        'nc' : 1.8240 , 
        'nd' : 1.8240 , 
        'ne' : 1.8240 , 
        'nf' : 1.8240 , 
        'nh' : 1.8240 , 
        'no' : 1.8240 , 
        's'  : 2.0000 , 
        's2' : 2.0000 , 
        's4' : 2.0000 , 
        's6' : 2.0000 , 
        'sx' : 2.0000 , 
        'sy' : 2.0000 , 
        'sh' : 2.0000 , 
        'ss' : 2.0000 , 
        'p2' : 2.1000 , 
        'p3' : 2.1000 , 
        'p4' : 2.1000 , 
        'p5' : 2.1000 , 
        'pb' : 2.1000 , 
        'pc' : 2.1000 , 
        'pd' : 2.1000 , 
        'pe' : 2.1000 , 
        'pf' : 2.1000 , 
        'px' : 2.1000 , 
        'py' : 2.1000 , 
        'f'  : 1.75   , 
        'cl' : 1.948  , 
        'br' : 2.02   , 
        'i'  : 2.15   , 
  }

# for lipid11
lipid_type_to_wdws = {
        'cA' : 1.9080 , 
        'cB' : 1.9080 , 
        'cC' : 1.9080 , 
        'oC' : 1.6612 , 
        'oS' : 1.6837 , 
        'oH' : 1.7210 , 
        'oT' : 1.6837 , 
        'oP' : 1.6612 , 
        'oO' : 1.6612 , 
        'nA' : 1.8240 , 
        'pA' : 2.1000 , 
        'hA' : 1.4870 , 
        'hE' : 1.3870 , 
        'hX' : 1.1000 , 
        'hB' : 1.4590 , 
        'hN' : 0.6000 , 
        'hO' : 0.0000 , 
        'cR' : 1.9080 , 
        'cP' : 1.9080 , 
        'oR' : 1.7210 , 
        'hR' : 0.0000 , 
        'hS' : 1.3870 , 
}

type_to_vdws.update(gaff_type_to_vdws)
type_to_vdws.update(lipid_type_to_wdws)

c_replace_list = ['CA','CB','CC','CD','CK','CM','CN','CQ','CR','CV','CW',
                  'CY','CZ','CP','CS',
                  'CO']
n_replace_list = ['NA','N2','N*','NC','NB','NT','NY']

from forcefield.amberbase import ConverterBase
class Format2AmberBaseConverter(ConverterBase):

    """
    This class converts the Amber style parameter set into the CURP style data.
    """

    def __init__(self, topology, use_atomtype=True):
        ConverterBase.__init__(self, topology)
        self.__mol_info = []
        self.__residue_info = {}
        self.__nonbond_interact_table = None

        self.__atom_info     = {}
        self.__bond_info     = {}
        self.__angle_info    = {}
        self.__torsion_info  = {}
        self.__improper_info = {}
        self.__coulomb_info  = {}
        self.__vdw_info      = {}

        self.__use_atomtype = use_atomtype

    def convert(self, format_setting=None):
        # bonded interaction
        self._convert_residue()
        self._convert_atom()
        self._convert_bond()
        self._convert_angle()
        self._convert_torsion()
        self._convert_improper()
        self._convert_coulomb()
        self._convert_vdw()

        if format_setting is None:
            return

        if format_setting.dump_parameters:
            self.print_atom()
            self.print_residue()
            self.print_bond()
            self.print_angle()
            self.print_torsion()
            self.print_improper()
            self.print_coulomb()
            self.print_vdw()
            self.print_bonded14_pairs()
            self.print_bonded_pairs()

    def get_mol_info(self):
        if not self.__mol_info:
            self._store_mol_info()
        return self.__mol_info

    def get_residue_info(self):
        return self.__residue_info

    def get_atom_info(self):
        return self.__atom_info

    def get_bond_info(self):
        return self.__bond_info

    def get_angle_info(self):
        return self.__angle_info

    def get_torsion_info(self):
        return self.__torsion_info

    def get_improper_info(self):
        return self.__improper_info

    def get_coulomb_info(self):
        return self.__coulomb_info

    def get_vdw_info(self):
        return self.__vdw_info

    def get_pbc_info(self):
        tpl = self.get_topology()
        box_info = tpl.get_param_infos('BOX_DIMENSIONS')
        return box_info

    def _store_mol_info(self):
        mol_info = self.get_topology().get_molcule_info()
        for name, nmol in zip(mol_info['mol_names'], mol_info['num_molecules']):
            natom = len(self.get_topology().get_atom_info(name)['names'])
            new_info = dict(name=name, nmol=nmol, natom=natom)
            self.__mol_info.append(new_info)

    def get_natom(self):
        tpl = self.get_topology()
        atom_names = tpl.get_param_infos('ATOM_NAME')
        return len(atom_names)

    def _convert_residue(self):

        ids       = []
        names     = []
        nres_last = 0
        natom = self.get_natom()

        tpl = self.get_topology()
        amb_res_ptrs = tpl.get_param_infos('RESIDUE_POINTER')
        amb_resnames = tpl.get_param_infos('RESIDUE_LABEL')

        # append the end pointer for each residue
        def gen_detailed_pointer(res_ptrs, natom):
            nres = len(amb_res_ptrs)
            for ires_1 in range(nres):
                iatm_beg = res_ptrs[ires_1]
                if ires_1+1 >= nres:
                    iatm_end = natom
                else:
                    iatm_end = res_ptrs[ires_1+1] - 1
                yield iatm_beg, iatm_end

        name_dptrs = zip(amb_resnames, gen_detailed_pointer(amb_res_ptrs,natom))
        for ires_1, (rname, res_dptr) in enumerate(name_dptrs):
            iatm_beg, iatm_end = res_dptr
            for iatm in range(iatm_beg, iatm_end+1):
                ids.append( ires_1 + 1 )
                names.append( rname )

        self.__residue_info = dict(ids=ids, names=names)

    def _convert_atom(self):
        tpl = self.get_topology()
        names       = tpl.get_param_infos('ATOM_NAME')
        masses      = tpl.get_param_infos('MASS')
        atom_types  = tpl.get_param_infos('AMBER_ATOM_TYPE')

        from math import pi

        elems = []
        for aname in names:
            if aname[0].isdigit():
                elems.append( aname[1] )
            else:
                elems.append( aname[0] )

        if self.__use_atomtype:
            vdw_radii = []
            for i_1, tname in enumerate(atom_types):
                radius = type_to_vdws.get(tname)
                if radius is not None:
                    # print('ok', tname)
                    vdw_radii.append( radius )
                else:

                    if tname in c_replace_list:
                        # print('C* ok')
                        vdw_radii.append( type_to_vdws['C*'] )

                    elif tname in n_replace_list:
                        vdw_radii.append( type_to_vdws['N'] )

                    elif tname[0].isdigit() and tname[1] == 'C':
                        vdw_radii.append( type_to_vdws['C*'] )

                    elif tname[1].isdigit() and tname[0] == 'C':
                        vdw_radii.append( type_to_vdws['C*'] )

                    elif tname[1].isdigit() and tname[0] == 'c':
                        vdw_radii.append( type_to_vdws['C*'] )

                    else:
                        # print('ng', tname)
                        type_to_vdws[tname]

        else:
            vdw_radii = []

        ids = range(1, self.get_natom()+1)
        self.__atom_info = dict(ids=ids, names=names, masses=masses, 
                elems=elems, vdw_radii=vdw_radii)

    def _convert_bond(self):

        tpl = self.get_topology()
        amb_consts    = tpl.get_param_infos('BOND_FORCE_CONSTANT')
        amb_eqs       = tpl.get_param_infos('BOND_EQUIL_VALUE')
        amb_atoms_hyd = tpl.get_param_infos('BONDS_INC_HYDROGEN')
        amb_atoms_woh = tpl.get_param_infos('BONDS_WITHOUT_HYDROGEN')

        two_atoms    = []
        force_consts = []
        length_eqs   = []
        ncol = 3

        # for bonds including hydrogen
        for ibnd in range( len(amb_atoms_hyd)/3 ):
            iatoms  = [ amb_atoms_hyd[ncol*ibnd+i]/3 + 1 for i in range(ncol-1) ]
            index = amb_atoms_hyd[ncol*(ibnd+1) - 1]

            two_atoms.append( iatoms )
            force_consts.append( amb_consts[index-1] )
            length_eqs.append( amb_eqs[index-1] )

        # for bonds without hydrogen
        for ibnd in range( len(amb_atoms_woh)/3 ):
            iatoms = [ amb_atoms_woh[ncol*ibnd+i]/3 + 1 for i in range(ncol-1) ]
            index = amb_atoms_woh[ncol*(ibnd+1) - 1]

            two_atoms.append( iatoms )
            force_consts.append( amb_consts[index-1] )
            length_eqs.append( amb_eqs[index-1] )

        self.__bond_info = dict(
                two_atoms=two_atoms,
                force_consts=force_consts,
                length_eqs=length_eqs )

    def _convert_angle(self):

        tpl = self.get_topology()
        amb_consts    = tpl.get_param_infos('ANGLE_FORCE_CONSTANT')
        amb_eqs       = tpl.get_param_infos('ANGLE_EQUIL_VALUE')
        amb_atoms_hyd = tpl.get_param_infos('ANGLES_INC_HYDROGEN')
        amb_atoms_woh = tpl.get_param_infos('ANGLES_WITHOUT_HYDROGEN')

        three_atoms  = []
        force_consts = []
        theta_eqs    = []
        ncol = 4

        # for angles including hydrogen
        for iang in range( len(amb_atoms_hyd)/ncol ):
            iatoms = [ amb_atoms_hyd[ncol*iang+i]/3 + 1 for i in range(ncol-1) ]
            index = amb_atoms_hyd[ncol*(iang+1) - 1]

            three_atoms.append( iatoms )
            force_consts.append( amb_consts[index-1] )
            theta_eqs.append( amb_eqs[index-1] )

        # for angles without hydrogen
        for iang in range( len(amb_atoms_woh)/4 ):
            iatoms = [ amb_atoms_woh[ncol*iang+i]/3 + 1 for i in range(ncol-1) ]
            index = amb_atoms_woh[ncol*(iang+1) - 1]

            three_atoms.append( iatoms )
            force_consts.append( amb_consts[index-1] )
            theta_eqs.append( amb_eqs[index-1] )

        self.__angle_info = dict(
                three_atoms=three_atoms,
                force_consts=force_consts,
                theta_eqs=theta_eqs )

    def _convert_torsion(self):
        self.__torsion_info = self.__convert_torsion(lambda i: i >= 0)

    def _convert_improper(self):
        self.__improper_info = self.__convert_torsion(lambda i: i < 0)

    def __convert_torsion(self, if_fun):

        tpl = self.get_topology()
        amb_consts    = tpl.get_param_infos('DIHEDRAL_FORCE_CONSTANT')
        amb_freqs     = tpl.get_param_infos('DIHEDRAL_PERIODICITY')
        amb_phases    = tpl.get_param_infos('DIHEDRAL_PHASE')
        amb_atoms_hyd = tpl.get_param_infos('DIHEDRALS_INC_HYDROGEN')
        amb_atoms_woh = tpl.get_param_infos('DIHEDRALS_WITHOUT_HYDROGEN')

        four_atoms     = []
        num_torsions   = []
        num_freqs      = []
        force_consts   = []
        initial_phases = []
        ncol = 5

        # for torsion including hydrogen
        for itor in range( len(amb_atoms_hyd)/ncol ):
            if not if_fun(amb_atoms_hyd[ncol*itor+3]): continue
            iatoms = [ abs(amb_atoms_hyd[ncol*itor+i])/3 + 1
                    for i in range(ncol-1) ]
            index = amb_atoms_hyd[ncol*(itor+1) - 1]

            four_atoms.append( iatoms )
            force_consts.append( amb_consts[index-1])
            num_torsions.append( 1 )
            num_freqs.append( amb_freqs[index-1] )
            initial_phases.append( amb_phases[index-1] )

        # for torsion without hydrogen
        for itor in range( len(amb_atoms_woh)/ncol ):
            if not if_fun(amb_atoms_woh[ncol*itor+3]): continue
            iatoms = [ abs(amb_atoms_woh[ncol*itor+i])/3 + 1
                    for i in range(ncol-1) ]
            index = amb_atoms_woh[ncol*(itor+1) - 1]

            four_atoms.append( iatoms )
            force_consts.append( amb_consts[index-1])
            num_torsions.append( 1 )
            num_freqs.append( amb_freqs[index-1] )
            initial_phases.append( amb_phases[index-1] )

        all_torsion_info = dict(
                four_atoms=four_atoms,
                num_torsions=num_torsions,
                num_freqs=num_freqs,
                force_consts=force_consts,
                initial_phases=initial_phases )

        return all_torsion_info

    def _convert_coulomb(self):

        tpl = self.get_topology()
        # divede by 18.2223 to convert to charge in units of the electron charge
        amb_charges = [c/18.2223 for c in tpl.get_param_infos('CHARGE')]
        self.__coulomb_info = dict(charges=amb_charges)

    def _convert_vdw(self):

        tpl = self.get_topology()
        amb_types   = tpl.get_param_infos('ATOM_TYPE_INDEX')
        amb_indexes = tpl.get_param_infos('NONBONDED_PARM_INDEX')
        amb_c12s    = tpl.get_param_infos('LENNARD_JONES_ACOEF')
        amb_c6s     = tpl.get_param_infos('LENNARD_JONES_BCOEF')
        import math
        ntype = int(math.sqrt(len(amb_indexes)))

        c6s  = numpy.zeros((ntype, ntype))
        c12s = numpy.zeros((ntype, ntype))

        for itype_1 in range(ntype):
            itype = itype_1 + 1
            for jtype_1 in range(ntype):
                jtype = jtype_1 + 1

                ij = ntype*(itype-1) + jtype
                index = amb_indexes[ij-1]
                # if index < 0: print('negative', index, amb_c6s[index-1])
                if index < 0: continue
                c6s[itype_1, jtype_1]  = amb_c6s[index-1]
                c12s[itype_1, jtype_1] = amb_c12s[index-1]

        self.__vdw_info = dict(atom_types=amb_types, c6s=c6s, c12s=c12s)

if __name__ == '__main__':
    logger.log_level = logger.INFO
    # tpl = TopologyParser('./test/ala3.prmtop')
    fp = "./test/b2ar-act.prmtop.gz"
    
    tpl = TopologyParser(fp)
    tpl.parse()

    converter = Format2AmberBaseConverter(tpl)
    converter.convert()

    # natom_target = 1467
    # target_atoms = range(1, natom_target+1)
    # converter.apply_target_atoms(target_atoms)

    converter.print_atom()
    # converter.print_residue()
    # converter.print_bond()
    # converter.print_angle()
    # converter.print_torsion()
    # converter.print_improper()
    # converter.print_coulomb()
    # converter.print_vdw()
    # converter.print_bonded14_pairs()
    # converter.print_bonded_pairs()

    # converter.print_ibnd_to_ipair()
    # converter.print_iang_to_ipair()
    # converter.print_itor_to_ipair()
    # converter.print_iimp_to_ipair()
    # converter.print_i14_to_ipair()

    # converter.print_nonbonded_table()



#! /usr/bin/env python2
from __future__ import print_function

import sys, os

def main():

    opts = parse_options()

    tc_fp1   = opts.tc_fps[0]
    tc_fp2   = opts.tc_fps[1]
    tc_outfp = opts.output_fp

    pair_to_tc1 = { (don,acc):tc for don,acc,tc,rest in load_tcdata(tc_fp1) }
    pair_to_tc2 = { (don,acc):tc for don,acc,tc,rest in load_tcdata(tc_fp2) }

    ipair_to_pair1 = { (get_id(don),get_id(acc)):(don, acc)
            for (don,acc) in pair_to_tc1.keys() }

    ipair_to_pair2 = { (get_id(don),get_id(acc)):(don, acc)
            for (don,acc) in pair_to_tc2.keys() }

    ipair_to_tc1 = { (get_id(don),get_id(acc)):tc
            for (don,acc), tc in pair_to_tc1.items() }
    ipair_to_tc2 = { (get_id(don),get_id(acc)):tc
            for (don,acc), tc in pair_to_tc2.items() }

    ipair1 = set(ipair_to_pair1)
    ipair2 = set(ipair_to_pair2)

    ipair_only1 = ipair1 - ipair2
    ipair_only2 = ipair2 - ipair1

    if opts.fill_pairs:
        all_ipair   = ipair1 | ipair2
        # ipair_to_pair
        ipair_to_pair = {}
        ipair_to_pair.update(ipair_to_pair1)
        ipair_to_pair.update(ipair_to_pair2)

        with open(tc_outfp, 'wb') as out_fd:
            fmt = "{:>12} {:>12} {:>20}\n"
            sorted_ipair = sorted(list(all_ipair))
            diff_iter = gen_diff_all(
                    sorted_ipair, ipair_to_tc1, ipair_to_tc2, opts.dval)
            for idon, iacc, tc in diff_iter:
                don, acc = ipair_to_pair[(idon,iacc)]
                out_fd.write(fmt.format(don, acc, tc))

    else:
        ok_ipair    = ipair1 & ipair2

        with open(tc_outfp, 'wb') as out_fd:
            fmt = "{:>12} {:>12} {:>20}\n"
            sorted_ipair = sorted(list(ok_ipair))
            diff_iter = gen_diff(sorted_ipair, ipair_to_tc1, ipair_to_tc2)
            for idon, iacc, tc in diff_iter:
                don, acc = ipair_to_pair1[(idon,iacc)]
                out_fd.write(fmt.format(don, acc, tc))

    fmt = "    {:>12} {:>12}"
    print("** Below pairs exist only in {}.".format(tc_fp1))
    for idon, iacc in sorted(list(ipair_only1)):
        don, acc = ipair_to_pair1[(idon,iacc)]
        print(fmt.format(don, acc))
    print("")

    print("** Below pairs exist only in {}.".format(tc_fp2))
    for idon, iacc in sorted(list(ipair_only2)):
        don, acc = ipair_to_pair2[(idon,iacc)]
        print(fmt.format(don, acc))
    print("")

    print("** finished completely. **")

def parse_options():
    """Parse and get the command line options."""

    # make argument parser
    import argparse
    parser = argparse.ArgumentParser(
        description= (
            'calculate difference between two energy conductivity files.\n'+
            'Result(that is written into output_tcdiff_file) = tc_file2 - tc_file1'
            ) )

    # add argument definitions
    parser.add_argument(
            'tc_fps', nargs='+', #action='append',
            help='specify filenames. ')

    parser.add_argument(
            '-o', '--output-filename', dest='output_fp', required=True,
            help='specify a filename to write the difference data.')

    parser.add_argument(
            '-f', '--fill-deficit-pairs', dest='fill_pairs', default=False,
            required=False, action='store_true',
            help='fill the deficit pairs.')

    parser.add_argument(
            '-d', '--deficit-value', dest='dval',
            type=float, required=False, default=10**(-8),
            help='the value to be set for deficit pairs if set -f.')

    # make arguments
    options = parser.parse_args()
    return options



def load_tcdata(fp):
    if fp.endswith('.gz'):
        import gzip
        fd = gzip.open(fp, 'rb')
    else:
        fd = open(fp, 'rb')

    lines = ( line for line in fd
            if not line.startswith('#')
            if not line.isspace() )

    for line in lines:
        cols = line.split()
        if len(cols) == 3:
            don, acc, tc = cols[0], cols[1], float(cols[2])
            rest = None
        elif len(cols) >= 4:
            don, acc, tc, rest = cols[0], cols[1], float(cols[2]), cols[3]

        yield don, acc, tc, rest

    fd.close()

def get_id(name):
    return name.split('_')[0]

def gen_diff(target_ipair, dict1, dict2):
    """Get dict2 - dict1."""
    for ipair in target_ipair:
        tc1 = dict1[ipair]
        tc2 = dict2[ipair]
        idon, iacc = ipair
        yield idon, iacc, tc2-tc1

def gen_diff_all(all_ipair, dict1, dict2, val):
    """Get dict2 - dict1 for all."""
    for ipair in all_ipair:
        tc1 = dict1.get(ipair, val)
        tc2 = dict2.get(ipair, val)
        idon, iacc = ipair
        yield idon, iacc, tc2-tc1


if __name__ == '__main__':
    main()




import os, sys
import traceback
import itertools as it
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)

################################################################################
from exception import WurpException

class FormatNotFoundError(WurpException): pass
class WriterNotFoundError(WurpException): pass
class NotFoundDictionary(WurpException): pass

################################################################################
class Writer:

    def __init__(self, trj_fp, trj_fmt,
            dt=0.01, is_vel=False, fst_lst_int=(1,-1,1)):

        fst, lst, intvl = fst_lst_int
        lst = 10**18 if lst == -1 else lst
        self.__fst_lst_int = (fst, lst, intvl)

        self.dt = dt

        if is_vel:
            Writer = _get_writer(trj_fmt, 'velocity')
        else:
            Writer = _get_writer(trj_fmt, 'coordinate')

        self.__writer = Writer(trj_fp, )
        self.__cnt = 0

    def write_header(self):
        self.__writer.write_header()

    def write(self, istp_1, trj, box=None):

        istp = istp_1+1
        fst, lst, intvl = self.__fst_lst_int

        if istp < fst: return
        if istp > lst: return

        if istp == fst:
            self.__writer.write(istp, trj, istp*self.dt, box)
            self.__cnt = 0
            return

        # if fst < istp < lst
        self.__cnt += 1

        if self.__cnt == intvl:
            self.__writer.write(istp, trj, istp*self.dt, box)
            self.__cnt = 0

    def close(self):
        self.__writer.close()


def _get_writer(format, writer_name):
    dictname = writer_name + '_writer_dict'

    # get the parsar's directory
    writer_path = os.path.dirname(__file__)

    # get the plugin names
    modpaths = []
    for fd in os.listdir(writer_path):
        abspath = os.path.join(writer_path, fd)
        if os.path.isdir(abspath) and not fd.startswith('_'):
            modpaths.append(abspath)

    for mpath in modpaths:
        modname = os.path.split(mpath)[-1]
        try:
            module = load_module(modname, writer_path)
            if hasattr(module, dictname):
                dictionary = getattr(module, dictname)
            else:
                raise NotFoundDictionary(dictname)

            if format in dictionary:
                Writer = dictionary[format]
                break

        except ImportError:
            print(traceback.format_exc())
            # raise FormatNotFoundError(format)

        except:
            raise

    else:
        raise WriterNotFoundError(format)

    return Writer

# class TrajectoryWriter:

    # def __init__(self, base_fp, writer, rotate=1, digit=3)
        # self.__base_fp = base_fp
        # self.__rotate  = rotate
        # self.__writer  = writer
        # self.__digit   = digit

        # self.__irot = 0


    # def write(self, snap):
        # self.__body.write(snap)

import imp
def load_module(modname,  basepath):
    f,n,d = imp.find_module(modname, [basepath])
    return imp.load_module(modname, f, n, d)


if __name__ == '__main__':

    Writer = _get_coordinate_writer('netcdf')
    print(Writer)





#! /usr/bin/env python
from __future__ import print_function
import os, sys
import itertools as it
import numpy
import pygraphviz as pgv

# FONTNAME = 'Helvetica Neue Bold'
FONTNAME = "Arial Bold"

class FormatWrongError(Exception): pass


def graph_een(data_fns, targets=['1-'], force_nodes=[], close_pairs=[],
              threshold=None, tpl_threshold=None, use_decrease=False,
              use_1letter=False, title='',
              ratio=None, alpha=False, graph_size=None, direction='TB',
              hide_isonode=False, line_values=[0.015, 0.008,0.003],
              line_colors=['red', 'blue', 'green'],
              line_thicks=[4.0, 4.0, 2.5], line_weights=[5.0, 3.0, 1.0],
              cluster_fn='', node_fn='', fig_fn=None, **kwds):

    if tpl_threshold is None:
        tpl_threshold = threshold

    # prepare
    parsers = [ EnergyConductivityParser(fp) for fp in data_fns ]

    # determine threshold value
    if threshold is None:
        threshold = line_values[-1]
    else:
        if threshold >= line_values[-1]:
            threshold = threshold
        else:
            threshold = line_values[-1]

    # communication chart
    cc  = CommunicationChart(
        title       = title if title else parsers[-1].get_title(),
        direction   = direction,
        threshold   = threshold,
        use_1letter = use_1letter,
        ratio       = ratio,
        alpha       = alpha,
        size        = graph_size,
        # font_scale  = 3.0 if opts.hide_node else 1.0,
        tpl_threshold = tpl_threshold,
        use_decrease = use_decrease)

    cc.set_line_thresholds(line_values,
            line_colors, line_thicks, line_weights)

    igrp_max = max([parser.get_max_id() for parser in parsers])
    igrp_min = min([parser.get_min_id() for parser in parsers])
    print('the range of id: {} - {}'.format(igrp_min, igrp_max))

    # donor and acceptor targets
    from wurp.table.target import parse_target_atoms_line as parse_target_group
    if targets:
        targets = parse_target_group(targets, igrp_max)
        cc.set_targets(targets)

    if force_nodes:
        force_nodes = parse_target_group(force_nodes, igrp_max)
        cc.set_force_nodes(force_nodes)

    # if don_targets or opts.acc_targets:
        # don_targets = parse_target_group(opts.don_targets, igrp_max)
        # acc_targets = parse_target_group(opts.acc_targets, igrp_max)
        # cc.restrict_groups(don_targets, acc_targets)

    # traverse all parsers
    values = []
    for parser in parsers:
        vs = list(parser)
        values += vs
    vs_1 = vs # for parsers[-1]

    # create nodes and edges from all the data for defining graph topology
    # but not showing edge shapes.
    if hide_isonode:
        all_nodes = []
        for v in values:
            don, acc, cond = v[0], v[1], v[2]
            all_nodes.append(don)
            all_nodes.append(acc)
            cc.add_data(don, acc, cond)

        target_nodes = []
        for v in vs_1:
            don, acc, cond = v[0], v[1], v[2]
            if cond >= threshold:
                nid1, nname1, label1 = cc._parse_label(don)
                nid2, nname2, label2 = cc._parse_label(acc)
                target_nodes.append(nname1)
                target_nodes.append(nname2)

        other_nodes = set(all_nodes) - set(target_nodes)
        for n in other_nodes:
            cc.hide_node(n)

        # if opts.output_layout_fp:
            # cc.write_layout(opts.output_layout_fp)

        for v in vs_1:
            don, acc, cond = v[0], v[1], v[2]
            cc.show_edge(don, acc, cond)

        #BEGUG
        # print( " ".join(str(rid) for rid in sorted(set(target_nodes)) ))

    else:
        # create nodes and edges from all the data for defining graph topology
        # but not showing edge shapes.
        for v in values:
            don, acc, cond = v[0], v[1], v[2]
            cc.add_data(don, acc, cond)

        # if opts.output_layout_fp:
            # cc.write_layout(opts.output_layout_fp)

        # draw edges for parser[-1] (data_fps[-1])
        for v in vs_1:
            don, acc, cond = v[0], v[1], v[2]
            cc.show_edge(don, acc, cond)

    # apply node style
    if node_fn:

        node_parser = NodeStyleParser(node_fn, (igrp_min, igrp_max))
        for secname in node_parser.get_secnames():
            nodes = node_parser.get_vals(secname)
            attr_dict = node_parser.get_attr_dict(secname)
            cc.apply_node_attributes(rids=nodes, nnames=[], **attr_dict)

    # apply node pairs to close in together
    if close_pairs:

        for word in close_pairs:
            rid1, rid2 = word.split(':')
            cc.apply_close_nodes(rid1, rid2)

    # read clusters file and apply clusters to graph if clusters file exists.
    if cluster_fn:

        cls_parser = ClusterParser(cluster_fn, (igrp_min, igrp_max))
        for secname in cls_parser.get_secnames():
            rids = cls_parser.get_vals(secname)
            cc.create_cluster(label=secname, rids=rids)

            attr_dict = cls_parser.get_attr_dict(secname)
            cc.apply_cluster_attributes(secname, **attr_dict)

        # g.graph_attr.update(color='pink')

    cc.draw(fig_fn)



class EnergyConductivityParser:

    def __init__(self, filename):

        self.parse(filename)

    def __iter__(self):
        return iter(self._data_iter)

    def parse(self, filename):
        self._title, self._labels = self.parse_header_lines(filename)
        self._data_iter = list(self.gen_parse_data(filename))

    def _open(self, filename):
        if filename.endswith('.gz'):
            fd = gzip.open(filename, 'rb')
        else:
            fd = open(filename, 'rb')

        return fd

    def parse_header_lines(self, filename):
        """Parse header line."""

        fd = self._open(filename)

        title = ''
        labels = []

        lines = ( line for line in fd if line.startswith('#') )

        for line, iline in it.izip(lines, range(100)):

            if line.startswith('#title'):
                title = line.replace('#title', '')

            elif line.startswith('#label'):
                labels = line.split()[1:]

            else:
                pass

        fd.close()

        return title, labels

    def gen_parse_data(self, filename):

        # get lines
        fd = self._open(filename)
        lines = ( line for line in fd if not line.startswith('#') )

        for line in lines:
            cols = line.split()
            don, acc = cols[0], cols[1]
            cond = float(cols[2])
            others = cols[3:]

            yield don, acc, cond, others

        fd.close()

    def get_max_id(self):

        maxid = 0
        for values in self._data_iter:
            acc_res = values[1]
            rid = int(acc_res.split('_')[0])
            maxid = max(maxid, rid)

        return maxid

    def get_min_id(self):

        minid = 99999
        for values in self._data_iter:
            don_res = values[0]
            rid = int(don_res.split('_')[0])
            minid = min(minid, rid)

        return minid

    def get_title(self):
        return self._title



class CommunicationChart:

    _res_3_to_1 = dict(
            ALA='A', ARG='R', ASN='N', ASP='D', CYS='C', GLN='Q', GLU='E',
            GLY='G', HIS='H', ILE='I', LEU='L', LYS='K', MET='M', PHE='F',
            PRO='P', SER='S', THR='T', TRP='W', TYR='Y', VAL='V', HIP='H',
            HID='H', HIE='H', ASH='D', GLH='E', CYX='C',
            )
    __res_3_to_1_main = {k+'M':v+'M' for k,v in _res_3_to_1.items()}
    __res_3_to_1_side = {k+'S':v+'S' for k,v in _res_3_to_1.items()}

    _res_3_to_1.update(__res_3_to_1_main)
    _res_3_to_1.update(__res_3_to_1_side)

    def __init__(self, title,direction='TB', ratio=None,
            use_1letter=False, alpha=None, size="", **options):
        # self.__graph = pgv.AGraph(label=title, rankdir=direction)

        # preprocess the graph options
        graph_opts = {}

        if size:
            graph_opts["ratio"] = "auto"
            graph_opts["size"]  = size
        else:
            graph_opts["ratio"] = ratio if ratio else "auto"

        if alpha is not None:
            graph_opts["bgcolor"] = "#000000{:02}".format(alpha)

        self.__graph = pgv.AGraph(label=title, rankdir=direction, **graph_opts)

        # self._ec_1 = self._ec_1_with_alpha
        # self._ec_2 = self._ec_2_with_alpha
        # self._ec_3 = self._ec_3_with_alpha
        # self._ec_4 = self._ec_4_with_alpha

        self.__opts = options

        self._rid_to_nname = {}
        self.__label_to_subgraph = {}
        self._subcount = 0
        self.__use_1letter = use_1letter
        # ratio = ratio if ratio else 'auto'

        self.__targets = []
        self.__force_nodes = []
        self.__don_targets = []
        self.__acc_targets = []
        self._edge_to_cond = {}

        # set default attributes for graph
        self.__graph.graph_attr.update(
                fontsize = '14'     ,
                fontname = FONTNAME,
                splines  = 'polyline' ,
                ratio    = ratio,
                # splines  = 'ortho'    ,
                # size = "100x500",
        )

        # set default attributes for nodes
        self.__graph.node_attr.update(
                shape     = 'box'   ,
                style     = 'rounded,filled' ,
                fontsize  = '14'  ,
                fontname  = FONTNAME,
                fixedsize = True   ,
                # height    = 0.02,
                # width     = 0.02,
                fillcolor = 'grey'  ,
                fontcolor = 'black' ,
                color     = 'black'  ,
        )

        # set default attributes for edges
        self.__graph.edge_attr.update(
                fontname = FONTNAME,
        )

    def set_line_thresholds(self, values, colors, thicks, weights):

        self._ec_thresholds = []
        for v, c, t, w in zip(values, colors, thicks, weights):
            self._ec_thresholds.append( (float(v), str(c), str(t), str(w)) )

        # for transparency
        # self._ec_thresholds.append( (0.015,   '#FF000080'   , '8.0' , '5' ) )
        # self._ec_thresholds.append( (0.008,   '#0000FF80'  , '4.5' , '3' ) )
        # self._ec_thresholds.append( (0.003,  '#00800080' ,   '2.0' , '1' ) )
        # self._ec_thresholds.append( (0.001,  '#00000080' , '0.5' , '1' ))

    def apply_node_attributes(self, rids=[], nnames=[], **attr_dict):
        """Apply givin attrifutes to nodes."""

        nnames_all = [ nname for nname in self.__graph.nodes() ]
        rids_all = self._rid_to_nname.keys()

        for rid in rids:
            if rid not in rids_all: continue

            nname = self._rid_to_nname[rid]
            node = self.__graph.get_node(nname)
            node.attr.update(**attr_dict)

        for nname in nnames:
            if nname not in nnames_all: continue

            node = self.__graph.get_node(nname)
            node.attr.update(**attr_dict)

    def apply_ec_attributes(self, **attributes):
        self.__graph.edge_attr.update( **attributes )

    def _parse_label(self, line):
        rid, rname = line.split('_')

        if self.__use_1letter:
            rname = self._res_3_to_1.get(rname.upper(), rname)

        if self._is_special(line):
            nname = self._get_spetial_nname(rname)
            label = self._get_spetial_label(rname)
        elif rname[-1].isdigit():
            nname = '{}_{}'.format(rid,rname)
            label = '{}'.format(rname)
        else:
            nname = '{}_{}'.format(rid,rname)
            label = '{}{}'.format(rname.title(), int(rid))

        return int(rid), nname, label

    def _get_spetial_nname(self, rname):
        fst = rname.find('(')
        lst = rname.find(')')
        return rname[fst+1:lst]

    def _get_spetial_label(self, rname):
        fst = rname.find('(')
        lst = rname.find(')')
        return rname[:fst]+rname[fst+1:lst]+rname[lst+1:]
        # return rname.replace('(','').replace(')','')

    def _get_edge_attrs(self, *values):

        cond = values[0]
        tau = values[1] if len(values)>=2 else ''

        for ec_thresh_info in self._ec_thresholds:
            if cond >= ec_thresh_info[0]:
                color, width, weight = ec_thresh_info[1:]
                break

        # if cond >= self._ec_1[0]:
            # color, width, weight = self._ec_1[1:]
        # elif  cond >= self._ec_2[0]:
            # color, width, weight = self._ec_2[1:]
        # elif cond >= self._ec_3[0]:
            # color, width, weight = self._ec_3[1:]
        # else:
            # color, width, weight = self._ec_4[1:]

        return color, width, weight, tau

    def add_data(self, don, acc, cond, *others):

        rid1, nname1, label1 = self._parse_label( don )
        rid2, nname2, label2 = self._parse_label( acc )

        # if it isn't target groups then dont't add nodes
        is_forced1 = nname1 in self.__force_nodes
        is_forced2 = nname2 in self.__force_nodes

        if not (is_forced1 or is_forced2):
            if not (nname1 in self.__targets or nname2 in self.__targets):
                return
            else:
                if nname1 not in self.__don_targets: return
                if nname2 not in self.__acc_targets: return

        if cond >= self.__opts['tpl_threshold']:
            self.__graph.add_node( nname1, label=label1, pin=True)
            self.__graph.add_node( nname2, label=label2, pin=True)
            self._rid_to_nname[rid1] = nname1
            self._rid_to_nname[rid2] = nname2
            self.add_edge(nname1, nname2, cond)

        elif self.__opts['use_decrease'] \
                and cond <= -self.__opts['tpl_threshold']:
            self.__graph.add_node( nname1, label=label1, pin=True)
            self.__graph.add_node( nname2, label=label2, pin=True)
            self._rid_to_nname[rid1] = nname1
            self._rid_to_nname[rid2] = nname2
            self.add_edge(nname1, nname2, -cond)

        elif is_forced1:
            self.__graph.add_node( nname1, label=label1, pin=True )
            self._rid_to_nname[rid1] = nname1

        elif is_forced2:
            self.__graph.add_node( nname2, label=label2, pin=True )
            self._rid_to_nname[rid2] = nname2

        else:
            pass

    # def remove_node(self, node):
        # nid, nname, label = self._parse_label( node )
        # if self.__graph.has_node(nname):
            # print('ok', nname)
            # self.__graph.remove_node( nname )

    def _is_special(self, label):
        fst = label.find('(')
        lst = label.find(')')

        if fst>=0 and lst>=0:
            if fst<lst: return True
            else: raise FormatWrongError(label)
        elif fst<0 and lst<0:
            return False
        else:
            raise FormatWrongError(label)

    def hide_node(self, node):
        rid, nname, label = self._parse_label( node )
        if self.__graph.has_node(nname):
            node = self.__graph.get_node( nname )
            node.attr['style'] = 'invis'

    def remove_edges_from(self, nodes):
        node_names = []
        for n in nodes:
            rid, nname, label = self._parse_label( n )
            node_names.append(nname)

        self.__graph.remove_edges_from( [node_names] )

    def add_edge(self, nname1, nname2, cond, *others):

        cond_old = self._edge_to_cond.get((nname1, nname2))
        cond_old_rev = self._edge_to_cond.get((nname2, nname1))

        if cond_old:
            if cond > cond_old:
                self._edge_to_cond[key] = cond
                color, width, weight, edge_label = self._get_edge_attrs(cond)
                self.__graph.add_edge( nname1, nname2,
                        weight=weight, style='invis' )

        elif cond_old_rev:
            if cond > cond_old_rev:
                self._edge_to_cond[key] = cond
                color, width, weight, edge_label = self._get_edge_attrs(cond)
                self.__graph.add_edge( nname2, nname1,
                        weight=weight, style='invis' )

        else:
            self._edge_to_cond[key] = cond
            color, width, weight, edge_label = self._get_edge_attrs(cond)
            self.__graph.add_edge( key[0], key[1],
                    weight=weight, style='invis' )

        # if cond_old:
            # if cond > cond_old:
                # self._edge_to_cond[key] = cond
                # color, width, weight, edge_label = self._get_edge_attrs(cond)
                # self.__graph.add_edge( key[0], key[1],
                        # weight=weight, style='invis' )

        # else:
            # self._edge_to_cond[key] = cond
            # color, width, weight, edge_label = self._get_edge_attrs(cond)
            # self.__graph.add_edge( key[0], key[1],
                    # weight=weight, style='invis' )

    def apply_force(self):
        # if it isn't target groups then dont't add nodes
        is_forced1 = nname1 in self.__force_nodes
        is_forced2 = nname2 in self.__force_nodes

        if is_forced1:
            self.__graph.add_node( nname1, label=label1 )
            self._rid_to_nname[rid1] = nname1

        if is_forced2:
            self.__graph.add_node( nname2, label=label2 )
            self._rid_to_nname[rid2] = nname2

        # if not (is_forced1 or is_forced2):
            # if not (nname1 in self.__targets or nname2 in self.__targets):
                # return
            # else:
                # if nname1 not in self.__don_targets: return
                # if nname2 not in self.__acc_targets: return

    def show_edge(self, don, acc, cond, *others):

        thres = self.__opts['threshold']

        if -thres < cond < thres: return
        if cond < 0.0 and not self.__opts["use_decrease"]: return

        rid1, nname1, label1 = self._parse_label( don )
        rid2, nname2, label2 = self._parse_label( acc )

        if not self.__graph.has_edge(nname1, nname2):
            if not self.__graph.has_node(nname1):
                self.__graph.add_node(nname1, label=label1)
                self._rid_to_nname[rid1] = nname1
            if not self.__graph.has_node(nname2):
                self.__graph.add_node(nname2, label=label2)
                self._rid_to_nname[rid2] = nname2

            self.__graph.add_edge(nname1, nname2)

        edge = self.__graph.get_edge(nname1, nname2)

        node1 = self.__graph.get_node(nname1)
        node2 = self.__graph.get_node(nname2)
        node1.attr.update(self.__graph.node_attr)
        node2.attr.update(self.__graph.node_attr)
        node1.attr['label'] = label1
        node2.attr['label'] = label2

        # fscale = self.__opts['font_scale']
        # node1.attr['fontsize'] = fscale *

        if cond > 0.0:
            color, width, weight, edge_label = self._get_edge_attrs(cond)
            edge.attr['style'] = ''
            edge.attr['color'] = color
            edge.attr['width'] = width
            edge.attr['label'] = edge_label
            edge.attr['penwidth'] = width
        else:
            color, width, weight, edge_label = self._get_edge_attrs(-cond)
            edge.attr['style'] = 'dashed'
            edge.attr['color'] = color
            edge.attr['width'] = width
            edge.attr['label'] = edge_label
            edge.attr['penwidth'] = width

    def restrict_groups(self, don_targets, acc_targets):
        self.__don_targets = don_targets
        self.__acc_targets = acc_targets

    def set_targets(self, targets):
        self.__targets = targets

    def set_force_nodes(self, force_nodes):
        self.__force_nodes = force_nodes

    def create_cluster(self, label, rids, **attributes):
        """Create subgraph cluster by label and nodes."""

        # default attributes
        default_attributes = dict(
                fontsize = 24,
                style = 'rounded,filled',
                color = 'lightgrey', )

        default_attributes.update( **attributes )

        # if node exists in graph
        # united_nnames = list( set(self._rid_to_nname.keys()) & set(rids) )
        united_nnames = [ self._rid_to_nname[rid]
                for rid in rids if self._rid_to_nname.get(rid) ]

        # create new graph
        self._subcount += 1
        name = 'cluster' + str(self._subcount)
        # name = 'a' + str(self._subcount)
        g = self.__graph.subgraph(nbunch=united_nnames, # rank='same',
                name=name, label=label, **default_attributes)
        self.__label_to_subgraph[label] = g
        # g.graph_attr['rank'] = 'same'
        return g

    def apply_cluster_attributes(self, secname, **attributes):
        g = self.__label_to_subgraph[secname]
        g.graph_attr.update( **attributes )

    def write_layout(self, fp):
        self.__graph.layout(prog='dot')
        self.__graph.write(fp)

    def draw(self, filename):
        # print('create legend')
        # self.create_legend()
        print('do layout')
        # self.__graph.layout(prog='dot')
        # self.__graph.write('tmp.dot')
        # self.__graph.write('test1.dot')
        # self.__graph.write('test2.dot')
        print('do draw')
        self.__graph.draw(filename, prog='dot')

    def apply_close_nodes(self, rid1, rid2, weight=3):

        nname1 = self._rid_to_nname.get(rid1)
        nname2 = self._rid_to_nname.get(rid2)

        if nname1 is None: return
        if nname2 is None: return

        if self.__graph.has_edge(nname1, nname2):
            edge = self.__graph.get_edge(nname1, nname2)
            edge.attr['weight'] = weight
        else:
            self.__graph.add_edge( nname1, nname2,
                    weight=weight, style='invis' )

    def get_nodes(self):
        return sorted(self._rid_to_nname.values())

    def create_legend(self):
        """Create a legend using the function of the subgraph."""

        # default attributes
        default_attributes = dict(
                fontsize = 48,)
                # style = 'rounded,filled',)
                # color = 'grey', fillcolor='white')

        # default_attributes.update( **attributes )
        # label = """
    # <TABLE BORDER="0" CELLBORDER="1" CELLSPACING="0" CELLPADDING="4">
     # <TR>
      # <TD COLSPAN="2"><B>Legend</B></TD>
     # </TR>
     # <TR>
      # <TD>Foo</TD>
      # <TD><FONT COLOR="red">Foo</FONT></TD>
     # </TR>
     # <TR>
      # <TD>Bar</TD>
      # <TD BGCOLOR="RED"></TD>
     # </TR>
     # <TR>
      # <TD>Baz</TD>
      # <TD BGCOLOR="BLUE"></TD>
     # </TR>
     # <TR>
      # <TD>Test</TD>
      # <TD><IMG src="so.png" SCALE="False" /></TD>
     # </TR>
     # <TR>
      # <TD>Test</TD>
      # <TD CELLPADDING="4">
       # <TABLE BORDER="1" CELLBORDER="0" CELLSPACING="0" CELLPADDING="0">
        # <TR>
         # <TD BGCOLOR="Yellow"></TD>
        # </TR>
       # </TABLE>
      # </TD>
     # </TR>
    # </TABLE>

        # """

        # label = '''<
        # <font color="red">aaa</font> <font color="blue">bbb</font>
        # >'''
        label_key1 = '''<<table border="0" cellpadding="2" cellspacing="0" cellborder="0">
        <tr><td align="right" port="i1">item 1</td></tr>
        <tr><td align="right" port="i2">item 2</td></tr>
        <tr><td align="right" port="i3">item 3</td></tr>
        <tr><td align="right" port="i4">item 4</td></tr>
        </table>>'''

        label_key2 = '''<<table border="0" cellpadding="2" cellspacing="0" cellborder="0">
      <tr><td port="i1">&nbsp;</td></tr>
      <tr><td port="i2">&nbsp;</td></tr>
      <tr><td port="i3">&nbsp;</td></tr>
      <tr><td port="i4">&nbsp;</td></tr>
      </table>>'''

        self.__graph.add_node('key1', label=label_key1, color='white',
                fillcolor='white', fontsize=32)
        self.__graph.add_node('key2', label=label_key2)
        # self.__graph.add_node('key1:i1:e')
        # self.__graph.add_node('key2:i1:w')

        for n in self.__graph.iternodes():
            print(n)
        # self.__graph.add_node('key1:1', label='hoge1')
        # self.__graph.add_node('key2:1', label='', fillcolor='white')

        # self.__graph.add_node('key1:2', label='hoge2')
        # self.__graph.add_node('key2:2', label='', fillcolor='white')
        # print(self.__graph.get_node('key:i1'))


        # self.__graph.add_edge('key1', 'key2', style='dashed',
                # **default_attributes)
        # self.__graph.add_edge('key1', 'key2', style='dashed', color='blue',
                # penwidth=10, **default_attributes)

        self.__graph.add_edge('key1:2', 'key2:2', penwidth=10,
                **default_attributes)


        # if node exists in graph
        # label = 'hoge'
        # self.__graph.add_node( 'legend', label=label, **default_attributes )

        # create new graph
        # name = 'legend' + str(self._subcount)
        name = 'legend'
        # # name = 'a' + str(self._subcount)
        # print(label)
        # nodes = ['key1:1', 'key1:2', 'key2:1', 'key2:2']
        nodes = ['key1', 'key2']
        g = self.__graph.subgraph(nbunch=nodes, rank='sink',
                name=name, label='Legend', rankdir='LR')# **default_attributes)
        # self.__label_to_subgraph[name] = g
        # # g.graph_attr['rank'] = 'sink'
        # return g


from wurp.table.ini_parser import IniParser
class ClusterParser(IniParser):

    def __init__(self, filename, igrp_range, *args, **kwds):
        self._igrp_range = igrp_range
        IniParser.__init__(self, filename, *args, **kwds)

    def _gen_col(self, lines):
        groups = []
        for line in lines:
            if '=' in line: continue
            for col in line.split():

                if '-' in col:
                    groups.extend( self._parse_mask_with_range(col, groups) )

                else:
                    groups.append( int(col) )

        return groups

    def _parse_mask_with_range(self, mask, groups):

        cols = mask.split('-')

        if cols[0] != '' and cols[1] != '':
            igrp_beg, igrp_end = int(cols[0]), int(cols[1])

        elif cols[0] == '' and cols[1] != '': # -45
            igrp_end = int(cols[1])

            if len(groups) == 0:
                igrp_beg = self._igrp_range[0]
            else:
                igrp_beg = groups[-1] + 1

        elif cols[0] != '' and cols[1] == '': # 45-
            igrp_beg = int(cols[0])
            igrp_end = self._igrp_range[1]

        else:
            pass

        return range(igrp_beg, igrp_end+1)

# define NodeStyleParser from ClusterParser
NodeStyleParser = ClusterParser


if __name__ == '__main__':
    from console import arg_graph_een, exec_command

    parser = arg_graph_een()
    exec_command(parser)




from __future__ import print_function
import os
import math
import numpy


import gzip
class FluxParser:

    section_end = '%end'

    def __init__(self, filename):

        if filename.endswith('.gz'):
            file = gzip.open(filename, 'rb')
        else:
            file = open(filename, 'rb')

        self.__file = file

        # make the generator
        self.__gen_snapshot_lines = self.parse_content(file)

    def __next__(self):
        try:
            lines = self.__gen_snapshot_lines.next()
            don_acc_pairs, fluxes = self.parse_onesnap(lines)
        except StopIteration:
            self.__file.close()
            raise

        return don_acc_pairs, numpy.array(fluxes)

    next = __next__ # for python 2.x

    def __iter__(self):
        return self

    def parse_header(self):
        lines = []
        for line in self.__file:
            if line[0] == '%':
                lines.append(line)
            else:
                break

        self.__file.seek(0)

        for line in lines:
            if line.startswith('%label'):
                label_line = line.replace('%label', '')

        return label_line.split()[2:]

    def close(self):
        self.__file.close()

    def parse_content(self, file):
        gen_lines = self.gen_optimized_lines(file)
        for line in gen_lines:
            if not self.is_section(line): continue

            flagname = line[1:].strip()
            if flagname != 'data': continue

            yield list( self.gen_section_lines(gen_lines) )

    def parse_onesnap(self, lines):
        """Parse the lines for one snapshot."""
        don_acc_pairs = []
        fluxes = []
        for line in lines:
            cols = line.split()
            don_acc_pairs.append( (cols[0], cols[1]) )

            values = [float(c) for c in cols[2:]]
            fluxes.append(values)

        return don_acc_pairs, numpy.array(fluxes)

    def gen_optimized_lines(self, file):
        # first flag
        for line in file:
            if self.is_section(line):
                yield line
                break
            yield line

        for line in file:
            if self.is_section(line):
                yield self.section_end
                yield line
            yield line

        else:
            yield self.section_end

    def split_content(self, gen):
        lines = []
        for line in gen:
            if line.startswith('%'):
                pass

    def is_section(self, line):
        if line == self.section_end:
            return False
        elif line.startswith('%'):
            return True
        else:
            return False

    def gen_section_lines(self, gen):
        for line in gen:
            if line == self.section_end:
                break
            elif self.is_section(line):
                continue
            yield line


class FluxWriter:

    def __init__(self, don_name, acc_name, base_fn, dt, labels):
        dirpath, fname = os.path.split(base_fn)
        filename = '{don}_{acc}.{base_fn}'.format(
                don=don_name, acc=acc_name, base_fn=fname)

        self.__filepath = os.path.join(dirpath, filename)
        self.__dt = dt
        self.__fmt = ('{:>12.4f}' + len(labels)*' {:16.8e}')
        self.write_header(don_name, acc_name, labels)

    def write_header(self, don_name, acc_name, labels, natom=None):
        file = self.open(self.__filepath)
        file.write('#REMARK  Flux Data for Acceptor <== Donor\n')
        file.write('#REMARK  Acceptor = {}'.format(acc_name) + '\n')
        file.write('#REMARK  Donor    = {}'.format(don_name) + '\n')
        # file.write('#REMARK  The number of atoms = {}'.format(natom) +
        info_line = '#REMARK time'
        for l in labels:
            info_line += ' {label:>16}'.format(label=l)

        file.write(info_line + '\n')
        file.close()

    def write(self, istep, flux):
        time = istep * self.__dt
        file = self.open(self.__filepath)
        file.write( self.__fmt.format(time, *flux) + '\n')
        file.close()

    def open(self, filepath):
        if filepath.endswith('.gz'):
            file = gzip.open(filepath, 'ab')
        else:
            file = open(filepath, 'ab')
        return file

def gen_flux_from_parsers(filenames):
    for fn in filenames:
        parser = FluxParser(fn)
        print('reading the data from {}'.format(fn))
        for pairs, fluxes in parser:
            yield pairs, fluxes

def gen_target_pairs(donor_line, acceptor_line, don_acc_pairs):

    if donor_line != '':
        target_dons = [ don for don in donor_line if don != '' ]
    else:
        donors = [ don for don, acc in don_acc_pairs ]
        target_dons = sorted( set(donors), key=donors.index )

    if acceptor_line != '':
        target_accs = [ acc for acc in acceptor_line if acc != '' ]
    else:
        acceptors = [ acc for don, acc in don_acc_pairs ]
        target_accs = sorted( set(acceptors), key=acceptors.index )

    print('target_dons', target_dons)
    print('target_accs', target_accs)

    for don, acc in don_acc_pairs:
        if don not in target_dons and acc not in target_accs: continue
        yield don, acc

def write_summary(filename, don_acc_pairs, labels, fluxes, wtype):

    basename, ext = os.path.splitext(filename)
    filepath = basename + '_' + wtype + ext

    open_ = gzip.open if filepath.endswith('.gz') else open

    # write header
    with open_(filepath, 'wb') as file:
        print('writing the {} data into {}'.format(wtype, filepath))
        file.write('%title {} flux\n'.format(wtype))
        label_line = '% {:>10} {:>12}'.format('donor', 'acceptor')
        for l in labels:
            label_line += ' {:>12s}'.format(l)
        file.write(label_line + '\n')

        fmt = '{:>12} {:>12}' + ' {:12.7f}'*len(labels) + '\n'
        for (don, acc), flux in zip(don_acc_pairs, fluxes):
            file.write( fmt.format(don, acc, *flux))

def write_avg_rms(filename, don_acc_pairs, avg_fluxes, rms_fluxes):

    basename, ext = os.path.splitext(filename)
    filepath = basename + '_avg+rms' + ext

    open_ = gzip.open if filepath.endswith('.gz') else open

    # write header
    with open_(filepath, 'wb') as file:
        print('writing the {} data into {}'.format('avg+rms', filepath))
        file.write('%title average and rms of the flux\n')
        label_line = '% {:>10} {:>12} {:>12} {:>12}'.format(
                'donor', 'acceptor', 'average', 'rms')
        file.write(label_line + '\n')

        fmt = '{:>12} {:>12} {:12.7f} {:12.7f}' + '\n'
        for (don, acc), avg, rms in zip(don_acc_pairs, avg_fluxes, rms_fluxes):
            file.write( fmt.format(don, acc, avg[0], rms[0]))


def divide_flux(flux_fns, output_fn, dt=1.0, donor_line='', acceptor_line='',
         column_line='', **kwds):
    """Divide flux file.

    Divide the flux data of all into the flux data for each donor and
    acceptor.
    """

    # prepare
    one_parser = FluxParser(flux_fns[0])
    all_labels = one_parser.parse_header()
    don_acc_pairs, fluxes = one_parser.next()
    one_parser.close()
    nvalue = len(all_labels)

    # parse colums list. For example, '1,4,6,9'
    if column_line == '':
        icolums = range(1, nvalue+1)
    else:
        icolums = [ int(col) for col in column_line.split(',') ]

    print('icolums', icolums)

    labels = []
    for icol in icolums:
        label = all_labels[icol-1]
        labels.append( label )

    print('labels', labels)

    # create average fluxes and fluxes**2 array
    npair = len(don_acc_pairs)
    print('npair', npair)
    fluxes_sum  = numpy.zeros([npair, nvalue], numpy.float)
    fluxes2_sum = numpy.zeros([npair, nvalue], numpy.float)

    # get target pairs
    target_pairs = list(gen_target_pairs(donor_line, acceptor_line,
                                          don_acc_pairs))
    target_indexes = []
    for index, pair in enumerate(target_pairs):
        target_indexes.append(index)

    print('target_pairs', target_pairs)
    print('target_indexes', target_indexes)

    # make writers
    ipair_to_writer = []
    for don_name, acc_name in target_pairs:
        writer = FluxWriter(don_name, acc_name,
                            output_fn, float(dt), labels)
        ipair_to_writer.append( writer )

    # parse and write the energy flux
    parsers = gen_flux_from_parsers(flux_fns)
    for istep, (pairs, fluxes) in enumerate(parsers):
        fluxes_sum  += fluxes
        fluxes2_sum += fluxes*fluxes

        for index, writer in zip( target_indexes, ipair_to_writer ):
            writer.write( istep, fluxes[index] )

    # postprocessing
    # for writer in ipair_to_writer:
    #     writer.close()

    avg_fluxes = fluxes_sum / float(istep+1)
    avg_fluxes2 = fluxes2_sum / float(istep+1)
    rms_fluxes = numpy.sqrt( avg_fluxes2 - avg_fluxes**2 )

    if len(labels) == 1:
        write_avg_rms(output_fn, target_pairs, avg_fluxes, rms_fluxes)
    else:
        # calculate average fluxes
        write_summary(output_fn, target_pairs, labels, avg_fluxes, 'avg')
        write_summary(output_fn, target_pairs, labels, rms_fluxes, 'rms')

    print()
    print(80*'-')
    print('    Script that divide flux data finished completely.')
    print(80*'-')


if __name__ == '__main__':
    from wurp.console import arg_divide_flux, exec_command

    parser = arg_divide_flux()
    exec_command(parser)



#! /usr/bin/env python2
from __future__ import print_function
import os, sys
import numpy
import pylab as pl

def load_data(filename):
    with open(filename, 'rb') as file:

        don_acc_lines = []
        ecs = []

        lines = (line for line in file
                if not line.startswith('#')
                if not line.isspace() )

        for line in lines:
            cols = line.split()
            don_acc_lines.append( (cols[0], cols[1]) )
            ecs.append( float(cols[2]) )

    return don_acc_lines, ecs

if __name__ == '__main__':

    base_fn = sys.argv[1]
    rel_fns = sys.argv[2:]

    pl.figure(figsize=(10,6))


    base_lines, base_ecs = load_data(base_fn)
    base_ecs = numpy.array(base_ecs)

    markers = ['^', 'D', 'o']
    colors  = ['red', 'green', 'blue']

    for fn, color, marker in zip(rel_fns, colors, markers):
        rel_lines, rel_ecs = load_data(fn)
        rel_ecs = numpy.array(rel_ecs)

        xs = rel_ecs/base_ecs

        pl.plot(xs, alpha=0.7, linestyle='_',
                color=color, marker=marker, markersize=4)

        for i, (x, base_ec, rel_ec) in enumerate(zip(xs, base_ecs, rel_ecs)):
            if x >= 1.2 or x <= 0.7:
                print(base_lines[i], x, base_ec, rel_ec)


    pl.ylim((0,3.0))
    pl.ylabel(r'$\langle irEC_{ij} \rangle_x / \langle irEC_{ij} \rangle_{100} $')
    pl.xlabel(r'Residue pair number')
    # pl.legend(('x=10', 'x=20', 'x=50'))

    rel_fn = rel_fns[-1]
    name = os.path.splitext(os.path.basename(rel_fn))[0]

    pl.savefig('avg-'+name+'.png', format='png', dpi=300)
    
    # pl.show()



#! /usr/bin/env python2
from __future__ import print_function
import os
import sys

def parse_helix(filename):
    """
    %TM1  1 25
    %TM2  40 90
    .
    .
    .
    """

    file = open(filename, 'rb')
    lines = (line for line in file
            if line.startswith("%H")
            or line.startswith("%TM") )

            # if line.startswith("%TM")
            # if line.startswith("%H") )

    for line in lines:
        cols = line.split()
        name, ibeg, iend = cols[0].replace('%',''), cols[1], cols[2]
        yield int(ibeg), int(iend)

    file.close()

def main(ec_fn, helix_fn, arg_3=None, **kwds):

    ibeg_iend_pairs = list(parse_helix(helix_fn))

    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')

    for line in ec_file:
        if line.startswith('#'):
            print(line.strip())
            continue

        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]
        rid1 = int(don_line.split('_')[0])
        rid2 = int(acc_line.split('_')[0])

        if arg_3 == '1':
            if rid1+4 >= rid2:
                for ibeg, iend in ibeg_iend_pairs:
                    if ibeg<=rid1<=iend and ibeg<=rid2<=iend:
                        print(line.strip())

        else:
            if rid1+4 == rid2:
                for ibeg, iend in ibeg_iend_pairs:
                    if ibeg<=rid1<=iend and ibeg<=rid2<=iend:
                        print(line.strip())

    ec_file.close()


if __name__ == '__main__':
    from wurp.tool.sele.console import exec_command, arg_sel_helix
    parser = arg_sel_helix()
    exec_command(parser)



from __future__ import print_function

# standard module
import os, sys
import numpy
import time

# wurp module
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import clog as logger

################################################################################
class TwoBodyForceBase:

    def __init__(self, topology, setting=None):
        self.__tpl = topology
        self.__setting = setting
        self.__natom = self.__tpl.get_natom()
        self.__forces = None
        self.__ptype_to_energy = {}
        self.__ptype_to_forces = {}
        self.__ptype_to_displacement = {}

    def get_pottypes(self):
        return ['bond','angle','torsion','improper',
                'coulomb14','vdw14','coulomb','vdw']
    
    def set_module(self, module):
        self.__mod = module

    def get_module(self):
        return self.__mod

    def get_natom(self):
        return self.__natom

    def setup(self, interact_table, check=False):
        self.__interact_table = interact_table
        max_tbf = self.get_maxpair(interact_table)
        self._setup_init(max_tbf, check)

        self._setup_bond()
        self._setup_angle()
        self._setup_torsion()
        self._setup_improper()

        self._setup_coulomb14()
        self._setup_vdw14()
        self._setup_coulomb()
        self._setup_vdw()

    def cal_force(self, crd):
        # initialize
        self.initialize(crd)

        # calculate the bonded components.
        for ptype in ('bond','angle','torsion','improper','coulomb14','vdw14'):
            self.cal_bonded(ptype)

        # calculate the nonbonded components.
        for t in self.__interact_table:
            self.cal_coulomb(t)
            self.cal_vdw(t)

        return self.__forces

    def _setup_init(self, max_tbf, check=False):
        self.__mod.setup(natom=self.get_natom(), check=check,
                bonded_pairs = self.__tpl.get_bonded_pairs(), max_tbf=max_tbf)

    def _setup_bond(self):
        """Prepare the parameter for the bond calculation."""
        mod = self.__setup_bondtype('bond')
        mod.ibnd_to_itbf = self.__tpl.get_ibnd_to_ipair()

    def _setup_angle(self):
        """Prepare the parameter for the angle calculation."""
        mod = self.__setup_bondtype('angle')
        mod.iang_to_itbf = self.__tpl.get_iang_to_ipair()

    def _setup_torsion(self):
        """Prepare the parameter for the torsion calculation."""
        mod = self.__setup_bondtype('torsion')
        mod.itor_to_itbf = self.__tpl.get_itor_to_ipair()

    def _setup_improper(self):
        """Prepare the parameter for the improper torsion calculation."""
        mod = self.__setup_bondtype('improper')
        mod.itor_to_itbf = self.__tpl.get_iimp_to_ipair()

    def __setup_bondtype(self, btype_name):
        """Prepare the parameter for the calculations without coulomb and vdw.
        """
        info = getattr(self.__tpl, 'get_'+btype_name+'_info')()
        mod_type = getattr(self.__mod, btype_name)
        for key in info.keys():
            value = info[key]
            setattr(mod_type, key, value)

        if btype_name == 'torsion':
            logger.info('The number of torsions :', len(mod_type.four_atoms))

        if btype_name == 'improper':
            logger.info('The number of impropers :', len(mod_type.four_atoms))

        return mod_type

    def _setup_coulomb(self):
        """Prepare the parameter for the coulomb calculation."""
        coulomb = self.__mod.coulomb
        info = self.__tpl.get_coulomb_info()
        coulomb.charges = info['charges']
        coulomb.cutoff_length = self.__setting.wurp.coulomb_cutoff_length

    def _setup_vdw(self):
        """Prepare the parameter for the vdw calculation."""
        vdw = self.__mod.vdw
        info = self.__tpl.get_vdw_info()
        vdw.atom_types = info['atom_types']
        vdw.c6s        = info['c6s']
        vdw.c12s       = info['c12s']
        vdw.cutoff_length = self.__setting.wurp.vdw_cutoff_length

    def _setup_coulomb14(self):
        """Prepare the parameter for the coulomb calculation."""
        coulomb14 = self.__mod.coulomb14
        info = self.__tpl.get_coulomb_info()
        coulomb14.charges = info['charges']
        # return self.__tpl.get_i14_to_ipair()
        i14_to_itbf = self.__tpl.get_i14_to_ipair()

        # if len(self.__tpl.get_i14_to_ipair())==1:
            # i14_to_itbf = numpy.array([])
        # else:
            # i14_to_itbf = self.__tpl.get_i14_to_ipair()

        # coulomb14.setup(info['charges'], i14_to_itbf)
        coulomb14.i14_to_itbf = self.__tpl.get_i14_to_ipair()

    def _setup_vdw14(self):
        """Prepare the parameter for the vdw calculation."""
        vdw14 = self.__mod.vdw14
        info = self.__tpl.get_vdw_info()
        vdw14.atom_types = info['atom_types']
        vdw14.c6s        = info['c6s']
        vdw14.c12s       = info['c12s']

        # if len(self.__tpl.get_i14_to_ipair())==1:
            # i14_to_itbf = numpy.array([])
        # else:
        i14_to_itbf = self.__tpl.get_i14_to_ipair()

        # vdw14.setup(info['atom_types'],info['c6s'],info['c12s'], i14_to_itbf)
        # print(self.__tpl.get_i14_to_ipair())
        vdw14.i14_to_itbf = self.__tpl.get_i14_to_ipair()
        # print(vdw14.i14_to_itbf )

        logger.info( 'The number of 1-4 interactions', len(i14_to_itbf))

    def initialize(self, crd):
        self.__mod.initialize(crd)
        self.__forces   = numpy.zeros( [self.__natom, 3] )
        self.__ptype_to_energy = {}
        self.__ptype_to_forces = {}

    def cal_bonded(self, bond_type):
        """Calculate the pairwise forces using the bonded type modules.
        Types are bond, angle, torsion, improper, coulomb14 and vdw14.
        """
        mod = getattr(self.__mod, bond_type)
        mod.calculate()
        self.__forces += mod.forces

        # store energy and forces
        self.__ptype_to_energy[bond_type] = mod.energy
        self.__ptype_to_forces[bond_type] = mod.forces
        self.__ptype_to_displacement[bond_type] = mod.displacement

        #DEBUG
        # print('** {} forces **'.format(bond_type))
        # for iatm_1, f in enumerate(mod.forces):
            # print("{:5>} {:15.7f}{:15.7f}{:15.7f}".format(
                # iatm_1+1, f[0],f[1],f[2]))
        # print()

        # if bond_type == 'bond':
            # print('** {} pairwise forces **'.format(bond_type))
            # for ibnd_1, itbf in enumerate(mod.ibnd_to_itbf):
                # tbf = mod.tbforces[itbf-1]
                # iatm, jatm = mod.two_atoms[ibnd_1]
                # print("{:5>} {:5>} {:5>} {:15.7f}{:15.7f}{:15.7f}".format(
                    # ibnd_1+1, iatm,jatm, tbf[0],tbf[1],tbf[2]))
            # print()

        #DEBUG

        return dict(energy = mod.energy,
                    forces = mod.forces,
                    tbforces = mod.tbforces,
                    displacement = mod.displacement)

    def cal_coulomb(self, table):
        return self._cal_nonbond(table, 'coulomb')

    def cal_vdw(self, table):
        return self._cal_nonbond(table, 'vdw')

    def _cal_nonbond(self, table, pottype):
        """Calculate the pairwise forces using the bonded type modules.
        Types are coulomb and vdw.
        """

        mod = getattr(self.__mod, pottype)
        mod.calculate(table)
        energy = mod.energy.copy()
        forces = mod.forces.copy()
        displacement = mod.displacement

        # store energy, forces and distance
        if pottype not in self.__ptype_to_energy:
            self.__ptype_to_energy[pottype] = energy
        else:
            self.__ptype_to_energy[pottype] += energy

        self.__forces += forces
        if pottype not in self.__ptype_to_forces:
            self.__ptype_to_forces[pottype] = forces
        else:
            self.__ptype_to_forces[pottype] += forces

        self.__ptype_to_displacement[pottype] = displacement

        # get pairwise forces
        tbforces = mod.tbforces.copy()

        return dict(energy = mod.energy,
                    forces = mod.forces,
                    tbforces = tbforces,
                    displacement = displacement)

    def get_forces(self, pottype):
        """Return the calculated force with givin potential type."""
        return self.__ptype_to_forces[pottype]

    def get_energy(self, pottype):
        """Return the calculated energy."""
        return self.__ptype_to_energy[pottype]

    def output_energy(self):
        """Output the energy."""
        logger.info_cycle('    ** Output energy **')
        for pottype in self.get_pottypes():
            energy = self.get_energy(pottype)
            logger.info_cycle('    {:>10} : {:>}'.format(pottype, energy))
        logger.info_cycle()

    def output_force(self):
        logger.debug_cycle('    ** Output force **')
        for pottype in self.get_pottypes():
            logger.debug_cycle('    [ {} force ] '.format(pottype))

            force = self.get_forces(pottype)
            for iatm_1, f in enumerate(force):
                logger.debug_cycle(
                    '    {:>8} : {:>12.8f} {:>12.8f} {:>12.8f}'.format(
                    iatm_1+1, f[0], f[1], f[2]))

        logger.debug_cycle('    [ Total force ] '.format(pottype))
        for iatm_1, f in enumerate(self.__forces):
            logger.debug_cycle(
                '    {:>8} : {:>12.8f} {:>12.8f} {:>12.8f}'.format(
                iatm_1+1, f[0], f[1], f[2]))

        logger.debug_cycle()

    def output_bonded(self, results, pot_type):
        print("[{}]".format(pot_type))
        print("== Energy = ")
        print(results['energy'])
        print()

        print("== Forces ==")
        print(results['forces'])
        print()

        print('== Two-body forces ==')
        caltype_mod = getattr(self.get_module(), pot_type)
        caltype_mod.print_tbforce()
        print()

    # def write_bond(self):
    #     fmt = "{iatm} {jatm} {x} {y} {z}"
    #     tbfs = self.__mod.bond.tbforces
    #     two_atoms = self.__tpl.get_bond_info()['two_atoms']
    #     for two, tbf in zip(two_atoms, tbfs):
    #         iatm, jatm = two
    #         print(fmt.format(
    #             iatm=iatm, jatm=jatm, x=tbf[0], y=tbf[1],z=tbf[2]))

    # def write_energy(self):
    #     pass

    def output_nonbonded(self, results, pot_type, table):
        print("[{}]".format(pot_type))
        print("== Energy == ")
        print(results['energy'])
        print()

        print("== Forces ==")
        print(results['forces'])
        print()

        print('== Two-body forces ==')
        tbfs = results['tbforces']
        itbf = 0
        for iatm, jatm_beg, jatm_end in table:
            for jatm in range(jatm_beg, jatm_end+1):
                itbf += 1
                print(iatm, jatm, tbfs[itbf-1])

        print()

    def get_maxpair(self, interact_table):
        maxpair = 0
        for t in interact_table:
            npair = 0
            for iatm, jatm_beg, jatm_end in t:
                npair += jatm_end - jatm_beg + 1

            maxpair = max(maxpair, npair)

        return maxpair

class TwoBodyForce(TwoBodyForceBase):

    def __init__(self, topology, setting=None):
        TwoBodyForceBase.__init__(self, topology, setting)
        import lib_amberbase
        self.set_module(lib_amberbase)


if __name__ == '__main__':
    import numpy

    class Setting:
        class Wurp:
            coulomb_cutoff_length = 10.0
            vdw_cutoff_length = 10.0

        wurp = Wurp()


    import os, sys
    topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    if topdir not in sys.path:
        sys.path.insert(0, topdir)

    from forcefield.amberbase import ConverterBase

    class DummyTopology(ConverterBase):

        def __init__(self):
            ConverterBase.__init__(self, None)

        def get_mol_info(self): pass
        def convert(self): pass
        def get_residue_info(self): pass
        def get_pbc_info(self): pass

        def get_natom(self):
            return 5

        def get_bond_info(self):
            return dict(
                two_atoms    = [[1,2], [1,3], [4,5]],
                force_consts = [1.0, 1.5, 1.9],
                length_eqs   = [1.0, 1.2, 0.9] )

        def get_angle_info(self):
            return dict(
                three_atoms  = [[1,2,3], [2,3,4], [1,2,5]],
                force_consts = [1.5, 2.0, 3.0],
                theta_eqs    = [90.0, 105.0, 120.0] )

        def get_torsion_info(self):
            return dict(
                four_atoms     = [[1,2,3,4], [2,1,4,5]],
                num_torsions   = [1, 1],
                num_freqs      = [2, 5],
                force_consts   = [1.5, 1.0],
                initial_phases = [180.0, 0.0] )

        def get_improper_info(self):
            return dict(
                four_atoms     = [[1,2,3,4], [2,1,4,5]],
                num_torsions   = [1, 1],
                num_freqs      = [2, 5],
                force_consts   = [1.5, 1.0],
                initial_phases = [180.0, 0.0] )

        def get_coulomb_info(self):
            return dict(
                charges = [0.5, 0.1, 0.3, 0.01, -0.6] )

        def get_vdw_info(self):
            return dict(
                atom_types = [1, 2, 1, 3, 1],
                c6s = [[100.0, 50.0, 35.5],
                       [50.0, 200.0, 10.0],
                       [35.0, 10.0, 300.0]],
                c12s = [[300.0, 91.0, 25.0],
                       [91.0, 200.0, 15.0],
                       [25.0, 15.0, 100.0]] )

        def get_bonded14_pairs(self):
            return [[1,4],[2,5]]

    # setup
    tbcal = TwoBodyForce(DummyTopology(), Setting())
    tbcal.setup(max_tbf=10)

    # # initialize with coordinate
    crd = numpy.array(
        [[1.0, 2.0, 3.0],
         [2.0, 1.0, 1.5],
         [3.0, 3.0, 3.0],
         [0.0, 0.0, 0.0],
         [5.5, 5.1, 4.5]])
    tbcal.initialize(crd)

    # # calculate two-body forces and output its data
    l = ['bond', 'angle', 'torsion', 'improper', 'coulomb14', 'vdw14']
    # l = ['coulomb14', 'vdw14']
    for caltype in l:
        res = tbcal.cal_bonded(caltype)
        tbcal.output_bonded(res, caltype)

    table = [[1, 2, 5], [2, 3, 5], [3,4,5], [4, 5, 5] ]
    for caltype in ['coulomb', 'vdw']:
        res = getattr(tbcal, 'cal_'+caltype)(table)
        tbcal.output_nonbonded(res, caltype, table)

    # crd = numpy.array(
    #     [[2.0, 3.0, 4.1],
    #      [3.0, 2.0, 2.5],
    #      [4.0, 4.0, 4.0],
    #      [1.0, 1.0, 1.0],
    #      [9.0, 9.1, 9.2]])
    # tbcal.initialize(crd, check=True)
    # tbcal.setup()
    # results = tbcal.cal_bonded()
    # output(results)

    # from benchmarker import Benchmarker
    # with Benchmarker(width=20) as bm:

    #     with bm('setup'):
    #         crd = numpy.array(
    #             [[2.0, 3.0, 4.1],
    #              [3.0, 2.0, 2.5],
    #              [4.0, 4.0, 4.0],
    #              [1.0, 1.0, 1.0],
    #              [9.0, 9.1, 9.2]])
    #         tbcal.initialize(crd, check=False)
    #         tbcal.setup()

    #     with bm('bonded'):
    #         results = tbcal.cal_bonded()
    #         output(results)

    #     table = [[1, 2, 5], [2, 3, 3], [2, 5, 5]]
    #     with bm('coulomb'):
    #         results = tbcal.cal_coulomb(table)
    #         output(results)

    #     with bm('vdw'):
    #         results = tbcal.cal_vdw(table)
    #         output(results)




import current
import flux

def get_calculator(setting):
    """Return the calculator object by setting."""

    # get volume setting
    method = setting.wurp.method

    if method == 'momentum-current':
        obj = current.StressCurrentCalculator
    elif method == 'stress-flux':
        obj = flux.StressFluxCalculator
    elif method == 'energy-current':
        obj = current.EnergyCurrentCalculator
    elif method == 'energy-flux':
        obj = flux.EnergyFluxCalculator
    elif method == 'heat-flux':
        obj = flux.HeatFluxCalculator
    else:
        raise Exception

    return obj





from __future__ import print_function

import os, sys
import time
import numpy

# wurp modules
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path: sys.path.insert(0, topdir)
from  utility import TimeStore
import clog as logger

################################################################################
class CalculatorBase(TimeStore):

    def __init__(self):
        TimeStore.__init__(self)
        self.__base_table = None

    def cal_twobody_force(self):
        pass

    def prepare(self, topology, setting, target_atoms,
            gname_iatoms_pairs, interact_table):

        self.__tpl = topology
        self.__setting = setting

        # decide two-body force calculator.
        import twobody
        TwoBodyCalculator = twobody.get_calculator(setting.wurp.potential)
        self.__tbf = TwoBodyCalculator(topology, setting)
        self.__tbf.setup(interact_table, check=False)

        # get the number of atoms
        natom = self.get_tbforce().get_natom()

        # for target atoms
        self.__target_atoms = target_atoms

        # for group
        self.__gname_iatoms_pairs = gname_iatoms_pairs
        self.__gnames = [gname for gname, iatoms in gname_iatoms_pairs]

        # interaction table
        self.__interact_table = [ numpy.array(t) for t in interact_table]

        # make a table to convert the atom index into the group index.
        iatm_to_igrp = numpy.zeros([natom])
        for igrp_1, (gname, iatoms) in enumerate(self.__gname_iatoms_pairs):
            for iatm in iatoms:
                iatm_to_igrp[iatm-1] = igrp_1+1

        self.__iatm_to_igrp = iatm_to_igrp

    def get_tbforce(self):
        return self.__tbf

    def get_topology(self):
        return self.__tpl

    def get_table(self):
        return self.__base_table

    def get_setting(self):
        return self.__setting

    def get_gname_iatoms_pairs(self):
        return self.__gname_iatoms_pairs

    def get_groupnames(self):
        return self.__gnames

    def get_iatm_to_igrp(self):
        return self.__iatm_to_igrp

    def get_target_atoms(self):
        return self.__target_atoms

    def get_bonded_pairs(self):
        return self.__tpl.get_bonded_pairs()

    def get_interact_table(self):
        return self.__interact_table

class CurrentCalculator(CalculatorBase):

    def __init__(self):
        CalculatorBase.__init__(self)
        self.__volumes = None
        self.__gvolumes = None

    def prepare(self, *args, **kwds):
        CalculatorBase.prepare(self, *args, **kwds)

        import volume
        self.__volume_obj = volume.get_volume_calculator( self.get_topology(),
                self.get_setting(), self.get_target_atoms(),
                self.get_gname_iatoms_pairs() )

    def get_volume_obj(self):
        return self.__volume_obj

    def run(self, data):
        """Run the current calculations for all of the components."""

        cstep, (crd, vel, pbc) = data
        logger.debug_cycle('    calculating current values at step {} ...'
                .format(cstep))

        # get twobody force object
        tbcal = self.get_tbforce()
        tbcal.initialize(crd)

        # calculate volume
        t0 = time.time()
        volume  = self.cal_volume(crd)
        gvolume = self.cal_gvolume(len(self.get_groupnames()))
        self.store_time('volume', time.time()-t0)

        # print('[Volume]')
        # print(volume)

        # gather the atomic _current for each potential types.
        key_to_acurs = {} # current for atoms
        key_to_icurs = {} # current for inside region of the groups
        key_to_ocurs = {} # current for outside region of the groups

        # kinetic
        cur_atm, cur_inn, cur_out = self.cal_kinetic(vel)
        key_to_acurs['kinetic'] = cur_atm
        key_to_icurs['kinetic'] = cur_inn
        key_to_ocurs['kinetic'] = cur_out

        # bonded
        btypes = self.get_topology().get_decomp_list('bonded+')
        for btype in btypes:
            # bond type
            cur_atm, cur_inn, cur_out = self.cal_bonded(crd, btype)
            key_to_acurs[btype] = cur_atm
            key_to_icurs[btype] = cur_inn
            key_to_ocurs[btype] = cur_out

        # non-bonded
        cur_atm, cur_inn, cur_out = self.cal_coulomb(crd)
        key_to_acurs['coulomb'] = cur_atm
        key_to_icurs['coulomb'] = cur_inn
        key_to_ocurs['coulomb'] = cur_out

        cur_atm, cur_inn, cur_out = self.cal_vdw(crd)
        key_to_acurs['vdw'] = cur_atm
        key_to_icurs['vdw'] = cur_inn
        key_to_ocurs['vdw'] = cur_out

        # group current = inside current + outside current
        key_to_gcurs = {}
        for key in key_to_acurs.keys():
            key_to_gcurs[key] = key_to_icurs[key] + key_to_ocurs[key]

        return cstep, (key_to_acurs, key_to_gcurs, key_to_icurs, key_to_ocurs)

    def cal_volume(self, crd):
        self.__volumes = self.get_volume_obj().get_volume(crd) 
        return self.__volumes

    def get_volume(self):
        return self.__volumes

    def cal_gvolume(self, ngroup):
        self.__gvolumes = self.get_volume_obj().get_gvolume(ngroup) 
        return self.__gvolumes

    def get_gvolume(self):
        return self.__gvolumes


################################################################################
class FluxCalculator(CalculatorBase):
    def __init__(self):
        CalculatorBase.__init__(self)

    def run(self, data):
        """Run the flux calculations for all of the components."""

        cstep, (crd, vel, pbc) = data
        logger.debug_cycle('    calculating flux values at step {} ...'
                .format(cstep))

        # get twobody force object
        tbcal = self.get_tbforce()
        tbcal.initialize(crd)

        # gather the flux for each potential types.
        key_to_aflux = {} # flux for atoms
        key_to_gflux = {} # flux for group

        btypes = self.get_topology().get_decomp_list('bonded+')
        for btype in btypes:
            # check for the amount of improper torsion and torsion.
            # if btype in ['improper','torsion']: continue
            # bond type
            flux_atm, flux_grp = self.cal_bonded(crd, vel, btype)
            key_to_aflux[btype] = flux_atm
            key_to_gflux[btype] = flux_grp

        # non-bonded
        flux_atm, flux_grp = self.cal_coulomb(crd, vel)
        key_to_aflux['coulomb'] = flux_atm
        key_to_gflux['coulomb'] = flux_grp
        flux_atm, flux_grp = self.cal_vdw(crd, vel)
        key_to_aflux['vdw'] = flux_atm
        key_to_gflux['vdw'] = flux_grp
        # total for atom
        if flux_atm is not None:
            total_atm = numpy.zeros( key_to_aflux['vdw'].shape )

            for flux in key_to_aflux.values():
                total_atm += flux

        else:
            total_atm = None

        # total for group
        if flux_grp is not None:
            total_grp = numpy.zeros( key_to_gflux['vdw'].shape )

            for flux in key_to_gflux.values():
                total_grp += flux

        else:
            total_grp = None

        key_to_aflux['total'] = total_atm
        key_to_gflux['total'] = total_grp

        # write energy
        if self.get_setting().output.output_energy:
            self.get_tbforce().output_energy()
        # write forces
        if logger.is_debug():
            self.get_tbforce().output_force()

        return cstep, (key_to_aflux, key_to_gflux)




from __future__ import print_function

import os, sys
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)

# additional package

# wurp package

################################################################################
class TopologyParser:

    """
    >>> tpl = TopologyParser('./test/ala3.tpl.gz')
    >>> tpl.parse()
    >>> tpl.get_molcule_info()

    """

    section_end = 'TPL> END SECTION'
    comment = ';'

    def __init__(self, filename):
        self._filename = filename

    def get_molcule_info(self):
        """Get name dictionary, name, mol_names, and num_molecules."""
        return self.__mol_info

    def get_atom_info(self, molname=None):
        """Get dictionary of ids, names, types, type_ids, ren_names,
        res_ids, masses, vdw_radii, and charges by molname.
        """
        if molname is None:
            molname = self.get_molcule_info()[0]['name']
        return self.__atom_infos.get(molname)

    def get_bond_info(self, molname=None):
        """Get dictionary of two_atoms, force_consts, and length_eqs
        by molname.
        """
        if molname is None:
            molname = self.get_molcule_info()[0]['name']
        return self.__bond_infos.get(molname)

    def get_angle_info(self, molname=None):
        """Get dictionary of three_atoms, force_consts, and theta_eqs
        by molname.
        """
        if molname is None:
            molname = self.get_molcule_info()[0]['name']
        return self.__angle_infos.get(molname)

    def get_torsion_info(self, molname=None):
        """Get dictionary of four_atoms, force_consts, num_torsions, num_freqs,
        initial_phases, and nonbonded_flags by molname.
        """
        if molname is None:
            molname = self.get_molcule_info()[0]['name']
        return self.__torsion_infos.get(molname)

    def get_improper_info(self, molname=None):
        """Get dictionary of four_atoms, force_consts, num_torsions, num_freqs,
        initial_phases, and nonbonded_flags by molname.
        """
        if molname is None:
            molname = self.get_molcule_info()[0]['name']
        return self.__improper_infos.get(molname)

    def get_nonbond_info(self):
        """Get dictionary of atom_types, vdw_radii and vdw_depths."""
        return self.__nonbond_info

    def parse(self):
        tpl_file = open(self._filename, 'r')
        sections = self.split_content(tpl_file)
        tpl_file.close()

        atom_infos     = {}
        bond_infos     = {}
        angle_infos    = {}
        torsion_infos  = {}
        improper_infos = {}

        mol_info = self.parse_mol(sections['mol'])
        self.print_info(mol_info)
        self.__mol_info = mol_info

        for atom_lines in sections['atom']:
            info = self.parse_atom(atom_lines)
            atom_infos[info['molname']] = info
            self.print_info(info)
        self.__atom_infos = atom_infos

        for bond_lines in sections['bond']:
            info = self.parse_bond(bond_lines)
            bond_infos[info['molname']] = info
            self.print_info(info)
        self.__bond_infos = bond_infos

        for angle_lines in sections['angle']:
            info = self.parse_angle(angle_lines)
            angle_infos[info['molname']] = info
            self.print_info(info)
        self.__angle_infos = angle_infos

        for torsion_lines in sections['torsion']:
            info = self.parse_torsion(torsion_lines)
            torsion_infos[info['molname']] = info
            self.print_info(info)
        self.__torsion_infos = torsion_infos

        for improper_lines in sections['improper']:
            info = self.parse_torsion(improper_lines)
            improper_infos[info['molname']] = info
            self.print_info(info)
        self.__improper_infos = improper_infos

        nonbond_info = self.parse_nonbond(sections['nonbond'])
        self.print_info(nonbond_info)
        self.__nonbond_info = nonbond_info

    def split_content(self, tpl_file):

        atom_lines_list     = []
        bond_lines_list     = []
        angle_lines_list    = []
        torsion_lines_list  = []
        improper_lines_list = []

        gen_lines = self.gen_optimized_lines(tpl_file)
        sections = []
        for line in gen_lines:
            if not self.is_section(line): continue

            lines = list( self.gen_section_lines(gen_lines) )

            if line.startswith('TPL> MOLECULES'):
                self.print_section('[MOLECULES]', lines)
                mol_lines = lines

            elif line.startswith('TPL> ATOMS'):
                self.print_section('[ATOMS]', lines)
                atom_lines_list.append(lines)

            elif line.startswith('TPL> BONDS'):
                self.print_section('[BONDS]', lines)
                bond_lines_list.append(lines)

            elif line.startswith('TPL> ANGLES'):
                self.print_section('[ANGLES]', lines)
                angle_lines_list.append(lines)

            elif line.startswith('TPL> TORSIONS'):
                self.print_section('[TORSIONS]', lines)
                torsion_lines_list.append(lines)

            elif line.startswith('TPL> IMPROPER-TORSIONS'):
                self.print_section('[IMPROPER]', lines)
                improper_lines_list.append(lines)

            # elif line.startswith('TPL> FUNCTIONS'):
            #     self.print_section('FUNCTIONS', lines)
            #     function_lines = lines

            elif line.startswith('TPL> NONBONDS'):
                self.print_section('[NONBONDS]', lines)
                nonbond_lines = lines

            else:
                continue

        return dict(mol=mol_lines, 
                atom=atom_lines_list, bond=bond_lines_list,
                angle=angle_lines_list, torsion=torsion_lines_list,
                improper=improper_lines_list,
                # function=function_lines,
                nonbond=nonbond_lines )

    def gen_optimized_lines(self, lines):
        """Generate lines with the end of sections and without the comments."""

        # search first section marker
        for line in lines:
            line = line.split(self.comment)[0].strip()
            if self.is_section(line):
                yield line
                break

        # rest
        for line in lines:
            line = line.split(self.comment)[0].strip()
            if self.is_section(line):
                yield self.section_end
                yield line
            elif line != '':
                yield line
            else:
                continue
        else:
            yield self.section_end

    def is_section(self, line):
        if line == self.section_end:
            return False
        elif line.startswith('TPL>'):
            return True
        else:
            return False

    def gen_section_lines(self, gen):
        for line in gen:
            if line == self.section_end:
                break
            yield line

    def print_section(self, header, lines):
        logger.debug(header)
        for l in lines:
            logger.debug(l)

    def print_info(self, info):
        """Print the content of information."""
        if 'molname' in info:
            logger.debug('')
            logger.debug('*** MOLCULE NAME: ' + info['molname'] + ' ***')
            logger.debug('')

        for key, values in info.items():
            logger.debug(key)
            logger.debug(values)

    def parse_mol(self, lines):

        mol_names = []
        num_molecules = []

        for line in lines:
            cols = line.split()
            mol_names.append( cols[0] )
            num_molecules.append( int(cols[1]) )

        return dict(name=None,
                mol_names=mol_names, num_molecules=num_molecules )

    def parse_atom(self, lines):

        molname = lines[0]

        ids       = []
        names     = []
        types     = []
        type_ids  = []
        res_names = []
        res_ids   = []
        masses    = []
        vdw_radii = []
        charges   = []

        arrow_flag = False

        for line in lines[1:]:
            if arrow_flag:
                if not line.strip().endswith('->'):
                    arrow_flag = False
                continue
                
            cols = line.split()

            names.append(cols[0])
            types.append(cols[1])
            type_ids.append(cols[2])
            res_names.append(cols[3])
            res_ids.append(int(cols[4]))
            masses.append(float(cols[5]))
            vdw_radii.append(float(cols[6]))
            charges.append(float(cols[7]))

            if line.strip().endswith('->'):
                arrow_flag = True

        return dict(molname=molname,
                ids=ids, names=names, types=types, type_ids=type_ids,
                res_names=res_names, res_ids=res_ids, masses=masses,
                vdw_radii=vdw_radii, charges=charges )

    def parse_bond(self, lines):

        molname = lines[0]

        two_atoms    = []
        force_consts = []
        length_eqs   = []

        for line in lines[1:]:
            cols = line.split()

            two_atoms.append( [int(cols[icol]) for icol in range(2)] )
            force_consts.append( float(cols[2]) ) # unit: kcal/(mol*A*A)
            length_eqs.append( float(cols[3]) )    # unit: A

        return dict(molname=molname,
                two_atoms=two_atoms, force_consts=force_consts,
                length_eqs=length_eqs )

    def parse_angle(self, lines):

        molname = lines[0]

        three_atoms     = []
        force_consts  = []
        theta_eqs = []

        for line in lines[1:]:
            cols = line.split()

            three_atoms.append( [int(cols[icol]) for icol in range(3)] )
            force_consts.append( float(cols[3]) ) # kcal/(mol*rad*rda)
            theta_eqs.append( float(cols[4]) ) # degree

        return dict(molname=molname,
                three_atoms=three_atoms, force_consts=force_consts,
                theta_eqs=theta_eqs )

    def parse_torsion(self, lines):

        molname = lines[0]

        four_atoms     = []
        num_torsions   = []
        num_freqs      = []
        force_consts   = []
        initial_phases = []
        nonbonded_flags = []

        for line in lines[1:]:
            cols = line.split()

            four_atoms.append( [int(cols[icol]) for icol in range(4)] )
            force_consts.append( float(cols[4]) )
            num_torsions.append( int(cols[5]) )
            num_freqs.append( int(cols[6]) )
            initial_phases.append( float(cols[7]) )
            nonbonded_flags.append( int(cols[8]) )

        return dict(molname=molname,
                four_atoms=four_atoms, force_consts=force_consts,
                num_torsions=num_torsions, num_freqs=num_freqs,
                initial_phases=initial_phases, 
                nonbonded_flags=nonbonded_flags )

    def parse_nonbond(self, lines):

        atom_types = []
        vdw_radii  = []
        vdw_depths = []

        for line in lines:
            cols = line.split()
            if len(cols) != 7: continue

            atom_types.append( int(cols[0]) )
            vdw_radii.append( float(cols[3]) )
            vdw_depths.append( float(cols[4]) )

        return dict(title=None,
                atom_types=atom_types, vdw_radii=vdw_radii,
                vdw_depths=vdw_depths )


################################################################################

from forcefield.amberbase import ConverterBase
class Format2Amber99Converter(ConverterBase):

    """
    A data in PRESTO version 2 style topology => force field
    """

    def __init__(self, topology):
        ConverterBase.__init__(self, topology)
        self.__mol_info = []
        self.__residue_info = {}
        self.__nonbond_interact_table = None

        self.__bond_info     = {}
        self.__angle_info    = {}
        self.__torsion_info  = {}
        self.__improper_info = {}
        self.__coulomb_info  = {}
        self.__vdw_info      = {}

        self.__14bonded_pairs = None

    def convert(self):
        # bonded interaction
        self._convert_residue()
        self._convert_bond()
        self._convert_angle()
        self._convert_torsion()
        self._convert_improper()
        self._convert_coulomb()
        self._convert_vdw()

    def get_mol_info(self):
        if not self.__mol_info:
            self._store_mol_info()
        return self.__mol_info

    def get_residue_info(self):
        return self.__residue_info

    def get_bond_info(self):
        return self.__bond_info

    def get_angle_info(self):
        return self.__angle_info

    def get_torsion_info(self):
        return self.__torsion_info

    def get_improper_info(self):
        return self.__improper_info

    def get_coulomb_info(self):
        return self.__coulomb_info

    def get_vdw_info(self):
        return self.__vdw_info

    def _store_mol_info(self):
        mol_info = self.get_topology().get_molcule_info()
        for name, nmol in zip(mol_info['mol_names'], mol_info['num_molecules']):
            natom = len(self.get_topology().get_atom_info(name)['names'])
            new_info = dict(name=name, nmol=nmol, natom=natom)
            self.__mol_info.append(new_info)

    def get_natom(self):
        natom = 0
        for info in self.get_mol_info():
            natom += info['nmol'] * info['natom']
        return natom

    def convert_atoms(self, atoms_info):

        self.topology.store_params('coulomb', charges=charges)

    def _convert_residue(self):

        ids       = []
        names     = []
        nres_last = 0

        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            atom_info = self.get_topology().get_atom_info(minfo['name'])
            if atom_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):
                    names.extend(atom_info['res_names'])

                    ids.extend( resid + nres_last
                            for resid in atom_info['res_ids'] )
                    nres_last = ids[-1]

        self.__residue_info = dict(ids=ids, names=names)

    def _convert_bond(self):

        two_atoms    = []
        force_consts = []
        length_eqs   = []
        cur_natom = 0

        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            bnd_info = self.get_topology().get_bond_info(minfo['name'])
            if bnd_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):

                    for iatoms in bnd_info['two_atoms']:
                        two_atoms.append(
                                [iatoms[i]+cur_natom for i in range(2)] )

                    force_consts.extend(bnd_info['force_consts'])
                    length_eqs.extend(bnd_info['length_eqs'])

                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        self.__bond_info = dict(
                two_atoms=two_atoms,
                force_consts=force_consts,
                length_eqs=length_eqs )

    def _convert_angle(self):

        three_atoms  = []
        force_consts = []
        theta_eqs    = []
        cur_natom    = 0

        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            ang_info = self.get_topology().get_angle_info(minfo['name'])
            if ang_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):

                    for iatoms in ang_info['three_atoms']:
                        three_atoms.append(
                                [iatoms[i]+cur_natom for i in range(3)] )

                    force_consts.extend(ang_info['force_consts'])
                    theta_eqs.extend(ang_info['theta_eqs'])
                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        self.__angle_info = dict(
                three_atoms=three_atoms,
                force_consts=force_consts,
                theta_eqs=theta_eqs )

    def _convert_torsion(self):
        self.__torsion_info  = self.__convert_torsion(
                self.get_topology().get_torsion_info)

    def _convert_improper(self):
        self.__improper_info = self.__convert_torsion(
                self.get_topology().get_improper_info)

    def __convert_torsion(self, get_info_fun):

        four_atoms     = []
        num_torsions   = []
        num_freqs      = []
        force_consts   = []
        initial_phases = []
        cur_natom = 0

        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            tor_info = get_info_fun(minfo['name'])
            if tor_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):

                    for iatoms in tor_info['four_atoms']:
                        four_atoms.append(
                                [iatoms[i]+cur_natom for i in range(4)] )

                    num_torsions.extend(tor_info['num_torsions'])
                    num_freqs.extend(tor_info['num_freqs'])
                    force_consts.extend(tor_info['force_consts'])
                    initial_phases.extend(tor_info['initial_phases'])

                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        all_torsion_info = dict(
                four_atoms=four_atoms,
                num_torsions=num_torsions,
                num_freqs=num_freqs,
                force_consts=force_consts,
                initial_phases=initial_phases )

        return all_torsion_info

    def _convert_coulomb(self):

        charges   = []
        cur_natom = 0

        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            atom_info = self.get_topology().get_atom_info(minfo['name'])
            if atom_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):
                    charges.extend(atom_info['charges'])
                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        self.__coulomb_info = dict(charges=charges)

    def _convert_vdw(self):

        cur_natom = 0
        atom_types = []

        # atom_types
        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            atom_info = self.get_topology().get_atom_info(minfo['name'])
            if atom_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):
                    atom_types.extend(atom_info['type_ids'])
                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        # c6s and c12s
        """Get dictionary of atom_types, vdw_radii and vdw_depths."""
        nonbond_info = self.get_topology().get_nonbond_info()

        types_radii_depths = zip(
                nonbond_info['atom_types'],
                nonbond_info['vdw_radii'],
                nonbond_info['vdw_depths'] )

        ntype = len(nonbond_info['atom_types'])
        c6s =  numpy.zeros((ntype,ntype))
        c12s = numpy.zeros((ntype,ntype))
        
        import math
        for itype, rad_i, dep_i in types_radii_depths:
            for jtype, rad_j, dep_j in types_radii_depths:
                dep_ij = math.sqrt(dep_i * dep_j)
                rad_ij = rad_j + rad_j
                c6s[itype-1, jtype-1]  = 2.0 * dep_ij * rad_ij**6
                c12s[itype-1, jtype-1] = dep_ij * rad_ij**12

        self.__vdw_info = dict(atom_types=atom_types, c6s=c6s, c12s=c12s)

    def _make_14bonded_pairs(self):

        cur_natom = 0
        two_atoms = []

        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            tor_info = self.get_topology().get_torsion_info(minfo['name'])
            if tor_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):

                    atoms_flags = zip(
                            tor_info['nonbonded_flags'],
                            tor_info['four_atoms'] )

                    ####################### flag ###############################
                    two_atoms.extend(
                        [ (iatoms[0]+cur_natom, iatoms[3]+cur_natom) 
                            for flag, iatoms in atoms_flags if flag==1 ] )
                    ####################### flag ###############################

                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        self.__14bonded_pairs = two_atoms

    def get_14bonded_pairs(self):
        if self.__14bonded_pairs is None:
            self._make_14bonded_pairs()
        return self.__14bonded_pairs
    get_bonded14_pairs = get_14bonded_pairs


################################################################################
class Format2OPLSConverter(Format2Amber99Converter):

    def _convert_vdw(self):

        cur_natom = 0
        interact_table = None
        atom_types = []

        # atom_types
        # loop for the number of molcule kinds
        for minfo in self.get_mol_info():

            atom_info = self.get_topology().get_atom_info(minfo['name'])
            if atom_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):
                    atom_types.extend(atom_info['type_ids'])
                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        # c6s and c12s
        """Get dictionary of atom_types, vdw_radii and vdw_depths."""
        nonbond_info = self.get_topology().get_nonbond_info()

        types_radii_depths = zip(
                nonbond_info['atom_types'],
                nonbond_info['vdw_radii'],
                nonbond_info['vdw_depths'] )

        ntype = len(nonbond_info['atom_types'])
        c6s =  numpy.zeros((ntype,ntype))
        c12s = numpy.zeros((ntype,ntype))
        
        import math
        for itype, rad_i, dep_i in types_radii_depths:
            for jtype, rad_j, dep_j in types_radii_depths:
                dep_ij = math.sqrt(dep_i * dep_j)
                rad_ij = rad_j * rad_j
                c6s[itype-1, jtype-1]  = 4.0 * dep_ij * rad_ij**3
                c12s[itype-1, jtype-1] = 4.0 * dep_ij * rad_ij**6

        all_vdw_info = dict( atom_types=atom_types, c6s=c6s, c12s=c12s )

        logger.debug('*** vdw ***')
        for key, values in all_vdw_info.items():
            logger.debug('[{}]'.format(key))
            logger.debug(values)

        return all_vdw_info


if __name__ == '__main__':
     logger.log_level = logger.INFO
     tpl = TopologyParser('./test/pyp.tpl')
     tpl.parse()

     converter = Format2Amber99Converter(tpl)
     converter.convert()
     converter.print_residue()
     converter.print_bond()
     converter.print_angle()
     converter.print_torsion()
     converter.print_improper()
     converter.print_coulomb()
     converter.print_vdw()
     converter.print_bonded_pairs()
     converter.print_14bonded_pairs()



import os, sys
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)

from exception import WurpException

class FileNotReadError(WurpException): pass

import lib_parser
class ParserBase:

    def __init__(self, filename, natom):
        self.__mod.initialize(filename)
        self.__natom = natom

    def _set_module(self, module):
        self.__mod = module

    def __next__(self):
        crd, ierr = self.__mod.parse_next(self.__natom)
        if ierr != 0:
            self.__mod.finalize()
            raise StopIteration()
        else:
            return crd
    next = __next__ # for python 2.x

    def __iter__(self):
        return self


class CoordinateParser(ParserBase):

    def __init__(self, filename, natom):
        self._set_module(lib_parser.unformatted_coordinate)
        ParserBase.__init__(self, filename, natom)


class VelocityParser(ParserBase):

    def __init__(self, filename, natom):
        self._set_module(lib_parser.unformatted_velocity)
        ParserBase.__init__(self, filename, natom)


class RestartParser:

    def __init__(self, filename, natom):
        self.__mod = lib_parser.unformatted_restart
        self.__mod.initialize(filename)
        self.__natom = natom
        self.__mod.parse1()

    def parse(self):
        crd, ierr = self.__mod.parse_crd(self.__natom)
        if ierr != 0:
            self.__mod.finalize()
            mes = 'in reading coodinate'
            raise FileNotReadError(mes)

        vel, ierr = self.__mod.parse_vel(self.__natom)
        if ierr != 0:
            self.__mod.finalize()
            mes = 'in reading velocity'
            raise FileNotReadError(mes)

        return crd, vel


if __name__ == "__main__":

    # import lib_parser

    # help(lib_parser)

    rst_fn = "./test/pyp_md.rst"

    parser = RestartParser(rst_fn, 10779)
    crd, vel = parser.parse()
    print( crd)
    print( vel)



#! /usr/bin/env python
from __future__ import print_function
import sys
import numpy

class InvalidNumberPairs(Exception): pass

def write_tc(tcs, pairs, tcs_rms):
    """Write transport coefficient and dispersion to standard output."""
    for pair, tc, rms in zip(pairs, tcs, tcs_rms):
        don, acc = pair
        print('{:>12} {:>12}  {}  {}'.format(don, acc, tc, rms))

def load_tc(fp):
    fd = open(fp, 'rb')
    lines = (line for line in fd
            if not line.startswith('#')
            if not line.isspace())

    pairs = []
    ipair_to_tc = []
    for line in lines:
        cols = line.split()
        don, acc = cols[0], cols[1]
        tc = float(cols[2])

        pairs += [(don, acc)]
        ipair_to_tc += [tc]

    return pairs, ipair_to_tc

def summarize_tc(tc_fps, **kwds):
    """Summarize transport coefficient"""

    ipair_to_tc_ary = []
    npair = None
    for fp in tc_fps:
        pairs, ipair_to_tc = load_tc(fp)
        ipair_to_tc_ary += [ipair_to_tc]

        if npair is None: npair = len(pairs)

        if npair != len(pairs):
            msg = "Number of pairs must be {}, but got {} in {}"
            raise InvalidNumberPairs(msg
                    .format(npair, len(pairs), fp))

    tc_ary = numpy.array(ipair_to_tc_ary)

    tc_avg = numpy.average(tc_ary, 0)
    tc_rms = numpy.std(tc_ary, 0)

    write_tc(tc_avg, pairs, tc_rms)

if __name__ == '__main__':
    from wurp.console import arg_sum_tc, exec_command

    parser = arg_sum_tc()
    exec_command(parser)



from __future__ import print_function

import os, sys
from abc import abstractmethod, abstractproperty, ABCMeta
import numpy

# wurp package
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import table.interact_table as it
import clog as logger


class ConverterPrintable(object):

    def print_atom(self):
        logger.info('*** atom ***')
        info = self.get_atom_info()
        atom = zip(info['names'], info['elems'], 
                info['masses'], info['vdw_radii'])
        for iatm_1, (name, elem, mass, radius) in enumerate(atom):
            logger.info('{:>7d}  {:<4s} {:<4s} {:12.7f} {:12.7f}'.format(
                iatm_1+1, name, elem, mass, radius))
        logger.info()

    def print_residue(self):
        logger.info('*** residue ***')
        info = self.get_residue_info()
        res = zip(info['ids'], info['names'])
        for iatm_1, (rid, rname) in enumerate(res):
            logger.info('{:>7d} {:>5d} {:5s}'.format(iatm_1+1, rid, rname))
        logger.info()

    def print_bond(self):
        logger.info('*** bond ***')
        info = self.get_bond_info()
        bond = zip(info['two_atoms'], info['force_consts'], info['length_eqs'])
        for ibnd_1, (iatoms, force, eq) in enumerate(bond):
            logger.info('{:>5d} {:>15}  {:12.7f}  {:12.7f}'.format(
                ibnd_1+1, iatoms, force, eq))
        logger.info()

    def print_angle(self):
        logger.info('*** angle ***')
        info = self.get_angle_info()
        angle = zip(info['three_atoms'], info['force_consts'],info['theta_eqs'])
        for iang_1, (iatoms, force, eq) in enumerate(angle):
            logger.info('{:>5d} {:>20}  {:12.7f}  {:12.7f}'.format(
                iang_1+1, iatoms, force, eq))
        logger.info()

    def print_torsion(self):
        logger.info('*** torsion ***')
        info = self.get_torsion_info()
        self._print_torsion(info)

    def print_improper(self):
        logger.info('*** improper ***')
        info = self.get_improper_info()
        self._print_torsion(info)

    def _print_torsion(self, info):
        torsion = zip(info['four_atoms'], info['num_torsions'],
                info['num_freqs'], info['force_consts'], info['initial_phases'])
        for itor_1, (iatoms, ntor, freq, force, phase) in enumerate(torsion):
            logger.info('{:>5d} {:>30}  {} {} {:12.7f}  {:12.7f}'.format(
                itor_1+1, iatoms, ntor, freq, force, phase ))
        logger.info()

    def print_coulomb(self):
        logger.info('*** coulomb ***')
        info = self.get_coulomb_info()
        for iatm_1, charge in enumerate(info['charges']):
            logger.info('{:>7d}  {:12.7f}'.format(iatm_1+1, charge))
        logger.info()

    def print_vdw(self):
        logger.info('*** vdw ***')
        logger.info('{:>5} {:>5}{:>17}{:>17}'
                .format('itype','jtype','c6','c12'))
        info = self.get_vdw_info()
        ntype, tmp = info['c6s'].shape
        c6s = info['c6s']
        c12s = info['c12s']
        for itype_1 in range(ntype):
            for jtype_1 in range(ntype):
                logger.info('{:>5} {:>5}  {:15.9E}  {:15.9E}'.format(itype_1+1,
                    jtype_1+1, c6s[itype_1, jtype_1], c12s[itype_1, jtype_1]))
        logger.info('[atom_types]')
        for iatm_1, itype in enumerate(info['atom_types']):
            logger.info('{:>7}  {:>3}'.format(iatm_1+1, itype))
        logger.info()

    def print_bonded_pairs(self):
        bonded_pairs = self.get_bonded_pairs()
        logger.info('*** bonded pairs ***')
        self._print_pairs(self.get_bonded_pairs())

    def print_bonded14_pairs(self):
        logger.info('*** 14 bonded pairs ***')
        self._print_pairs(self.get_bonded14_pairs())

    def print_nonbonded_table(self):
        logger.info('*** non-bonded interaction table ***')

        interact_table = self.get_nonbonded_table()

        logger.info( '{:>10}{:>10}{:>10}'.format('iatm', 'jatm_beg', 'jatm_end') )
        for iatm, jatm_beg, jatm_end in interact_table:
            logger.info( '{:>10}{:>10}{:>10}'.format(iatm,jatm_beg,jatm_end) )

        logger.info()

    def print_ibnd_to_ipair(self): self._print_idx_to_ipair('bnd')
    def print_iang_to_ipair(self): self._print_idx_to_ipair('ang')
    def print_itor_to_ipair(self): self._print_idx_to_ipair('tor')
    def print_iimp_to_ipair(self): self._print_idx_to_ipair('imp')
    def print_i14_to_ipair(self):  self._print_idx_to_ipair('14')

    def _print_pairs(self, pairs):
        npair = len(pairs)
        logger.info(' The number of pairs = {}'.format(npair))
        logger.info('{:>7} {:>7} {:>7}'.format('ipair', 'iatm', 'jatm'))
        for ipair_1, (iatm, jatm) in enumerate(pairs):
            logger.info('{:>7d} {:>7d} {:>7d}'.format(ipair_1+1, iatm, jatm))
        logger.info()

    def _print_idx_to_ipair(self, bondtype):
        
        logger.info('*** i{} => ipair ***'.format(bondtype))
        get = getattr(self, 'get_i'+bondtype+'_to_ipair')
        idx_to_ipair = get()

        n = len(idx_to_ipair)
        logger.info(' The number of indices = {}'.format(n))

        if len(numpy.shape(idx_to_ipair)) == 1:
            ncol = 1
        else:
            ncol = len(idx_to_ipair[0])
        fmt = '{:>7}' + '{:>' + str(ncol*8) + '}'

        logger.info(fmt.format('index', 'ipairs'))
        for idx_1, ipairs in enumerate(idx_to_ipair):
            logger.info(fmt.format(idx_1+1, ipairs))
        logger.info()


import lib_bonded_pair
class TableMaker:

    def __init__(self):

        self._bonded_pairs    = None
        self._bonded14_pairs  = None
        self._nonbonded_table = None

        self._ibnd_to_ipair = None
        self._iang_to_ipair = None
        self._itor_to_ipair = None
        self._iimp_to_ipair = None
        self._i14_to_ipair  = None

        self._target_atoms = None

    def get_decomp_list(self, dtype='all'):
        """Get the list that decompose all potential.
        dtype = 'all'(defaulst), 'bonded', 'bonded14', 'bonded+',
        'nonbonded or 'nonbonded+(nonbonded+14)'
        """
        bonded_list    = ['bond', 'angle', 'torsion', 'improper']
        bonded14_list  = ['coulomb14', 'vdw14']
        nonbonded_list = ['coulomb', 'vdw']

        if   dtype == 'bonded':     return bonded_list
        elif dtype == 'bonded14':   return bonded14_list
        elif dtype == 'bonded+':    return bonded_list + bonded14_list
        elif dtype == 'nonbonded':  return nonbonded_list
        elif dtype == 'nonbonded+': return nonbonded_list + bonded14_list
        else: return bonded_list + bonded14_list + nonbonded_list

    def get_bonded_pairs(self):
        if self._bonded_pairs is None:
            self._bonded_pairs = self._make_bonded_pairs()
        return self._bonded_pairs

    def get_bonded14_pairs(self):
        if self._bonded14_pairs is None:
            self._bonded14_pairs = self._make_bonded14_pairs()
        return self._bonded14_pairs

    def get_nonbonded_table(self):
        if self._nonbonded_table is None:
            self._nonbonded_table = self._make_nonbonded_table()
        return self._nonbonded_table

    def get_ibnd_to_ipair(self): return self._get_idx_to_ipair('bnd')
    def get_iang_to_ipair(self): return self._get_idx_to_ipair('ang')
    def get_itor_to_ipair(self): return self._get_idx_to_ipair('tor')
    def get_iimp_to_ipair(self): return self._get_idx_to_ipair('imp')

    def get_i14_to_ipair(self):
        return self._get_idx_to_ipair('14')
        # if len(self._get_idx_to_ipair('14'))==1:
            # return numpy.array([])
        # else:
            # return self._get_idx_to_ipair('14')

    #TODO
    def apply_target_atoms(self, target_atoms):
        self.__target_atoms = target_atoms

        # bond
        info = self.get_bond_info()
        iatoms_list = info['two_atoms']
        self._apply_target_to_bondtype(iatoms_list, info)

        # angle
        info = self.get_angle_info()
        iatoms_list = info['three_atoms']
        self._apply_target_to_bondtype(iatoms_list, info)

        # torsion
        info = self.get_torsion_info()
        iatoms_list = info['four_atoms']
        self._apply_target_to_bondtype(iatoms_list, info)

        # improper
        info = self.get_improper_info()
        iatoms_list = info['four_atoms']
        self._apply_target_to_bondtype(iatoms_list, info)

        # bonded14
        self._make_bonded14_pairs()

        # bonded
        self._make_bonded_pairs()

    #TODO
    def _make_nonbonded_table(self):
        import time

        bonded_pairs = numpy.array(self.get_bonded_pairs())

        natom = self.get_natom()
        int_table = it.InteractionTable(self.get_natom())

        import lib_nonbond
        mod_tab = lib_nonbond.without_bonded
        mod_tab.setup(bonded_pairs, natom)

        int_table = numpy.array( list(int_table) )
        nonbonded_table, ntable = mod_tab.get_nonbonded_table( int_table )
        new_table = it.InteractionTable(
                base_table=nonbonded_table[:ntable].tolist())

        return new_table

    def _make_bonded_pairs(self):
        two_atoms   = self.get_bond_info()['two_atoms']
        three_atoms = self.get_angle_info()['three_atoms']
        four_atoms  = self.get_torsion_info()['four_atoms']
        four_atoms_imp  = self.get_improper_info()['four_atoms']

        import itertools as it
        pairs = [] # ipair => (iatm, jatm)
        for two in two_atoms:
            iatm, jatm = min(two), max(two)
            pairs += [(iatm, jatm)]

        for three in three_atoms:
            for pair in it.combinations(three, 2):
                iatm, jatm = min(pair), max(pair)
                pairs += [(iatm, jatm)]
                
        for four in four_atoms:
            for pair in it.combinations(four, 2):
                iatm, jatm = min(pair), max(pair)
                pairs += [(iatm, jatm)]

        for four in four_atoms_imp:
            for pair in it.combinations(four, 2):
                iatm, jatm = min(pair), max(pair)
                pairs += [(iatm, jatm)]

        return sorted(set(pairs), key=lambda x:x[0])

    def _make_bonded14_pairs(self):

        pairs = []
        four_atoms = self.get_torsion_info()['four_atoms']
        for four in four_atoms:
            pair = four[0], four[3]
            iatm, latm = min(pair), max(pair)
            pairs += [(iatm, latm)]

        three_pairs = []
        three_atoms = self.get_angle_info()['three_atoms']
        for three in three_atoms:
            pair = three[0], three[2]
            iatm, katm = min(pair), max(pair)
            three_pairs += [(iatm, katm)]

        return sorted(set(pairs)-set(three_pairs), key=lambda x:x[0])

    #TODO
    def _make_bonded14_pairs_old(self):

        cur_natom = 0
        two_atoms = []

        # loop for the number of the kinds for molcule
        for minfo in self.get_mol_info():

            tor_info = self.get_topology().get_torsion_info(minfo['name'])
            if tor_info:

                # loop for the number of molecules
                for imol in range( minfo['nmol'] ):

                    atoms_flags = zip(
                            tor_info['nonbonded_flags'],
                            tor_info['four_atoms'] )

                    two_atoms.extend(
                        [ (iatoms[0]+cur_natom, iatoms[3]+cur_natom) 
                            for flag, iatoms in atoms_flags if flag==1 ] )

                    cur_natom += minfo['natom']

            else:
                cur_natom += minfo['natom'] * minfo['nmol']

        self._bonded14_pairs = two_atoms

    def _get_idx_to_ipair(self, bonded_type):
        dictname = 'i'+bonded_type+'_to_ipair'
        to_ipair = getattr(self, '_'+dictname)
        if to_ipair is None:
            make_method = getattr(self, '_make_'+dictname)
            to_ipair = make_method()
        return to_ipair

    def _apply_target_to_bondtype(self, iatoms_list, info):

        target_atoms = self.__target_atoms

        new_infos = {key:[] for key in info.keys()}
        for i_1, iatoms in enumerate(iatoms_list):
            if not self._contains(target_atoms, iatoms): continue

            for key, new_info in new_infos.items():
                new_infos[key].append( info[key][i_1] )

        for key, new_info in new_infos.items():
            info[key] = new_info

    def _contains(self, target_atoms, iatoms):
        for iatm in iatoms:
            if iatm in target_atoms: return True
        else:
            return False

    def _make_ibnd_to_ipair(self):
        iatoms_list  = self.get_bond_info()['two_atoms']
        bonded_pairs = self.get_bonded_pairs()

        # make table
        ibnd_to_ipair = lib_bonded_pair.get_ibnd_to_ipair(
                numpy.array(iatoms_list), numpy.array(bonded_pairs) )

        return ibnd_to_ipair

    def _make_iang_to_ipair(self):
        iatoms_list  = self.get_angle_info()['three_atoms']
        bonded_pairs = self.get_bonded_pairs()

        # make table
        if len(iatoms_list) == 0:
            iang_to_ipair = numpy.array([[]])
        else:
            iang_to_ipair = lib_bonded_pair.get_iang_to_ipair(
                    numpy.array(iatoms_list), numpy.array(bonded_pairs) )

        return iang_to_ipair

    def _make_itor_to_ipair(self):
        iatoms_list  = self.get_torsion_info()['four_atoms']
        bonded_pairs = self.get_bonded_pairs()

        # make table
        if len(iatoms_list) == 0:
            itor_to_ipair = numpy.array([[]])
        else:
            itor_to_ipair = lib_bonded_pair.get_itor_to_ipair(
                    numpy.array(iatoms_list), numpy.array(bonded_pairs) )

        return itor_to_ipair

    def _make_iimp_to_ipair(self):
        iatoms_list  = self.get_improper_info()['four_atoms']
        bonded_pairs = self.get_bonded_pairs()

        # make table
        if len(iatoms_list) == 0:
            iimp_to_ipair = numpy.array([[]])
        else:
            iimp_to_ipair = lib_bonded_pair.get_itor_to_ipair(
                    numpy.array(iatoms_list), numpy.array(bonded_pairs) )

        return iimp_to_ipair

    def _make_i14_to_ipair(self):
        iatoms_list  = self.get_bonded14_pairs()
        bonded_pairs = self.get_bonded_pairs()

        # make table
        if len(iatoms_list) == 0:
            i14_to_ipair = numpy.array([[]])
        else:
            i14_to_ipair = lib_bonded_pair.get_ibnd_to_ipair(
                    numpy.array(iatoms_list), numpy.array(bonded_pairs) )

        return i14_to_ipair


class ConverterBase(ConverterPrintable, TableMaker):
    __metaclass__ = ABCMeta

    def __init__(self, topology):
        self.__topology = topology
        TableMaker.__init__(self)

    @abstractmethod
    def convert(self):
        """Convert program-specific data to the CURP data structure."""
        return

    @abstractmethod
    def get_natom(self):
        """Get the number of atoms."""
        return

    @abstractmethod
    def get_mol_info(self):
        """Get the molecular information."""
        return

    @abstractmethod
    def get_atom_info(self):
        """Get the dictionary of atom ids, names, masses, elems, vdw_radii."""
        return

    @abstractmethod
    def get_residue_info(self):
        """Get the dictionary of residue ids and names for each atom."""
        return

    @abstractmethod
    def get_bond_info(self):
        """Get bond dictionary of two_atoms, force_consts, and length_eqs."""
        return

    @abstractmethod
    def get_angle_info(self):
        """Get angle dictionary of three_atoms, force_consts, and theta_eqs."""
        return

    @abstractmethod
    def get_torsion_info(self):
        """Get torsion dictionary of four_atoms, num_torsions, num_freqs,
        force_consts, and initial_phases.
        """
        return

    @abstractmethod
    def get_improper_info(self):
        """Get improper torsion dictionary of four_atoms, num_torsions,
        num_freqs, force_consts, and initial_phases."""
        return

    @abstractmethod
    def get_coulomb_info(self):
        """Get coulomb dictionary of charges."""
        return

    @abstractmethod
    def get_vdw_info(self):
        """Get vdw dictionary of atom_types, c6s, c12s."""
        return

    @abstractmethod
    def get_pbc_info(self):
        """Get the box for beriodic boundary condition."""
        return

    def get_topology(self):
        return self.__topology





from __future__ import print_function

# standard modules
import os, sys
import time
import numpy

# wurp modules
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path: sys.path.insert(0, topdir)
from utility import TimeStore
import clog as logger

class NEVIntegrator(TimeStore):

    coef = 4.184*10.0**(-4) # dt*f/m (fs*kcal/mol/A/u) => v(A/fs)
    gas_const = 8.3144621 # gas constant (J/K/mol)

    def __init__(self, topology, setting, interact_table, crd=None, vel=None):
        TimeStore.__init__(self)
        self.setup(topology, setting, interact_table)
        self.__crd = crd
        self.__vel = vel

    def setup(self, topology, setting, interact_table):
        self.__tpl = topology
        self.__setting = setting

        # decide force calculator
        import twobody
        TwoBodyCalculator = twobody.get_calculator(setting.wurp.potential)
        self.__tbf = TwoBodyCalculator(topology, setting)
        self.__tbf.setup(interact_table, check=False)

        self.__interact_table = [ numpy.array(t) for t in interact_table ] 

        # decide integrator
        params = setting.dynamics
        if params.integrator == 'vverlet':
            self.integrate = self.do_vverlet
        elif params.integrator == 'leapfrog':
            self.integrate = self.do_leapfrog
        else:
            pass

    def run(self, data=None):
        """Run the one step integrator."""

        if data:
            cstep, (crd, vel, pbc) = data
        else:
            crd, vel = self.__crd, self.__vel

        logger.debug('    calculate dynamics ...')

        masses   = self.__tpl.get_atom_info()['masses']
        force = self.cal_force(crd)
        params = self.__setting.dynamics

        results = self.integrate(crd, force, vel, masses, params)

        return results

    def cal_force(self, crd):
        return self.__tbf.cal_force(crd)

    def integrate(self, crd, frc, vel, masses):
        pass
    
    def do_vverlet(self, crd, frc, vel, masses, params):
        """Integrate the coordinate and velocity according to
        velocity verlet algorithm.
        """

        # istp     : current step
        # crd      : current step's coordinate
        # vel      : current step's velocity
        # frc      : current step's force
        # crd_next : next step's coordinate
        # vel_next : next step's velocity
        # frc_next : next step's force

        # Preparation of the dynamcs on 0th step
        ms    = numpy.array(masses)[:, None]
        dt    = params.dt * 1000.0
        nstep = params.num_steps
        coef  = self.coef

        # log the information of the restart file
        logger.info('*** The information of the restart file ***')
        # log temperature
        ek, temp = self.cal_temp(vel, ms)
        self.output_temp(ek, temp)
        # log energy
        if self.__setting.output.output_energy: self.__tbf.output_energy()
        # log forces
        if logger.is_debug(): self.__tbf.output_force()

        # do dynamics
        for istp_1 in range(nstep):
            istp = istp_1+1

            t0 = time.time()

            logger.set_curstep(istp)
            logger.info_cycle('*** ISTEP = {}, CURRENT TIME = {} ***'
                .format(istp, istp * dt))

            # calculate next step's coordinate
            # r(t+dt) = r(t) + dt*v(t) + dt^2/2m * F(t)
            # crd_next = crd + dt*vel + 0.5*dt*dt * frc / ms * coef
            crd_next = crd + dt*vel + 0.5*dt*dt * frc / ms * coef
            
            # calculate next step's forces from coordinate
            frc_next = self.cal_force(crd_next)

            # calculate next step's velocity
            # v(t+dt) = v(t) + dt/2m * { F(t+dt) + F(t) }
            vel_next = vel + 0.5*dt* (frc_next+frc) / ms * coef

            self.store_time('integrator', time.time()-t0)
            yield istp, (crd_next, vel_next)

            crd = crd_next
            frc = frc_next
            vel = vel_next

            # log temperature
            ek, temp = self.cal_temp(vel, ms)
            self.output_temp(ek, temp)
            # log energy
            if self.__setting.output.output_energy: self.__tbf.output_energy()
            # log forces
            if logger.is_debug(): self.__tbf.output_force()

    def do_leapfrog(self, crd, frc, vel, masses, params):
        """Integrate the coordinate and velocity according to
        leap frog algorithm.
        """

        # istp     : current step(t)
        # crd      : current step(t)'s coordinate
        # frc      : current step(t)'s force
        # vel      : current step(t-dt/2)'s velocity
        # crd_next : next step(t+dt)'s coordinate
        # vel_next : next step(t+dt/2))'s velocity
        # frc_next : next step(t+dt)'s force
        # vel_dt = 0.5*(vel + vel_next)

        # Preparation of the dynamcs on 0th step
        ms    = numpy.array(masses)[:, None]
        # dt    = params.dt
        # dt    = params.dts
        dt = params.dt * 1000.0
        nstep = params.num_steps
        coef  = self.coef

        # log the information of the restart file
        logger.info('*** The information of the restart file ***')
        logger.info("    Velocity is 0+1/2-th step' informations")
        # log temperature
        ek, temp = self.cal_temp(vel, ms)
        self.output_temp(ek, temp)
        # log energy
        if self.__setting.output.output_energy: self.__tbf.output_energy()
        # log forces
        if logger.is_debug(): self.__tbf.output_force()

        # do dynamics
        for istp_1 in range(nstep):
            istp = istp_1+1

            t0 = time.time()

            logger.set_curstep(istp)
            logger.info_cycle('*** ISTEP = {}, CURRENT TIME = {} ***'
                .format(istp, istp * dt))

            # calculate next step's velocity
            # v(t+dt/2) = v(t-dt/2) + dt * F(t) /m
            vel_next = vel + dt*frc/ms * coef
            vel_t   = 0.5*( vel + vel_next )

            # calculate next step's coordinate
            # r(t+dt) = r(t) + dt*v(t+dt/2)
            crd_next = crd + dt*vel_next
            
            # calculate next step's forces from coordinate
            frc_next = self.cal_force(crd_next)

            self.store_time('integrator', time.time()-t0)
            yield istp, (crd_next, vel_next)

            crd = crd_next
            frc = frc_next
            vel = vel_next

            # log temperature
            ek, temp = self.cal_temp(vel_t, ms)
            self.output_temp(ek, temp)
            # log energy
            if self.__setting.output.output_energy: self.__tbf.output_energy()
            # log forces
            if logger.is_debug(): self.__tbf.output_force()

    def cal_temp(self, vel, ms):
        """Calculate the temperature and Kenitic energy."""
        gas  = self.gas_const
        dof  = vel.size # defree of freedom

        energy_tmp = numpy.sum( ms * vel**2)

        energy = 0.5 * energy_tmp / self.coef
        temp   = 10.0**7 * energy_tmp / dof / gas
        return energy, temp

    def output_temp(self, energy, temp):
        """Output the temperature and Kinetic energy into log file."""
        logger.info_cycle('    Temperature    : {:>8.2f} (K) '.format(temp))
        logger.info_cycle('    Kinetic energy : {:>8.5f} (K) '.format(energy))




#! /usr/bin/env python2
from __future__ import print_function

import sys


def parse_line(line):
    cols = line.split()
    don_line, acc_line, rest = cols[0], cols[1], cols[2:]

    rid_don, rname_don = don_line.split('_')
    rid_don = int(rid_don)
    rid_acc, rname_acc = acc_line.split('_')
    rid_acc = int(rid_acc)

    return rid_don, rid_acc, rname_don, rname_acc, rest


def main(renums, ec_fn=None, **kwds):
    """Renumber the residue numbers according to renums.

    Parameters
    ----------
    renums : iterable of strings
        Residue to renumber and which number to assign.

    Examples
    --------
    With an irEC file ec.dat so that:
    >>> with open('ec.dat', 'r') as ec_file:
    ...     print(ec_file.read())
    ...
        00001_ACE    00002_ILE  0.1  0.1
        00001_ACE    00004_PRO  0.2  0.2
        00004_PRO    00006_GLU  0.3  0.3

    Then to renumber the first residue to 3, and the fourth to 10:
    >>> main('ec.dat', ['1:3', '4:10'])
        00003_ACE    00004_ILE  0.1  0.1
        00003_ACE    00010_PRO  0.2  0.2
        00010_PRO    00012_GLU  0.3  0.3
    """

    # Remove the only space line and the comment line

    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')
    lines = [line.strip() for line in ec_file
             if not line.isspace() if not line.startswith('#')]

    # Obtain maximum residue number
    rid_max = 0
    for line in lines:
        rid1, rid2, rname1, rname2, rest = parse_line(line)
        rid_max = max(rid_max, rid1, rid2)

    new_rids = range(rid_max+1)
    old_rids = range(rid_max+1)

    # Make dictionary to alter the given residue identifiers
    for renum in renums:
        rid_old, rid_new = renum.split(':')
        rid_old, rid_new = int(rid_old), int(rid_new)

        shift = rid_new - rid_old
        new_rids[rid_old:] = [rid+shift for rid in old_rids[rid_old:]]

    # Convert residue identifiers into new ones
    for line in lines:
        rid1, rid2, rname1, rname2, rest = parse_line(line)
        ec = rest[0]

        fmt = '{:05}_{:<7}  '
        print(fmt.format(new_rids[rid1], rname1),
              fmt.format(new_rids[rid2], rname2), ec)


if __name__ == '__main__':
    from wurp.tool.console import exec_command, arg_renum_residue
    parser = arg_renum_residue()
    exec_command(parser)



import os, sys

topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)

from exception import WurpException
class PotentialNotFoundError(WurpException): pass

################################################################################
def get_calculator(forcefield):
    if forcefield in ('amber-base','amber94','amber96','amber99',
            'amber99SB','amber03', 'amber10', 'amber12SB'):
        import amberbase
        return amberbase.TwoBodyForce

    else:
        raise PotentialNotFoundError(forcefield)




#! /usr/bin/env python2
from __future__ import print_function

import os
import sys

def is_cap(rname):
    return rname.lower() in ['nme', 'ace']


def main(ec_fn, **kwds):
    if ec_fn is None:
        ec_file = sys.stdin
    else:
        ec_file = open(ec_fn, 'r')
    lines = (line for line in ec_file
             if not line.startswith('#')
             if not line.isspace())

    for line in lines:
        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]

        rname1 = don_line.split('_')[1]
        rname2 = acc_line.split('_')[1]
        if is_cap(rname1): continue
        if is_cap(rname2): continue

        print(line.strip())


if __name__ == '__main__':
    from wurp.tool.sele.console import exec_command, arg_sel_nocap
    parser = arg_sel_nocap()
    exec_command(parser)
